(self.webpackChunkcentreon=self.webpackChunkcentreon||[]).push([["6556"],{90035:function(e,t,n){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var r=n(27303),o=n(5580);function i(e,t){for(let n of e){let e=t.startsWith(n.name),r=t.replace(n.name,"");if(e){if(r.startsWith("/")){let e=n.name;return{pkgNameOrAlias:e,expose:r=`.${r}`,remote:n}}if(""===r)return{pkgNameOrAlias:n.name,expose:".",remote:n}}let o=n.alias&&t.startsWith(n.alias),i=n.alias&&t.replace(n.alias,"");if(n.alias&&o){if(i&&i.startsWith("/")){let e=n.alias;return{pkgNameOrAlias:e,expose:i=`.${i}`,remote:n}}if(""===i)return{pkgNameOrAlias:n.alias,expose:".",remote:n}}}}function a(e,t){for(let n of e)if(t===n.name||n.alias&&t===n.alias)return n}function s(e,t){let n=r.getGlobalHostPlugins();return n.length>0&&n.forEach(t=>{(null==e?void 0:e.find(e=>e.name!==t.name))&&e.push(t)}),e&&e.length>0&&e.forEach(e=>{t.forEach(t=>{t.applyPlugin(e)})}),e}function l(){return(l=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e}).apply(this,arguments)}async function u({entry:e,remoteEntryExports:t}){return new Promise((n,r)=>{try{t?n(t):Function("callbacks",`import("${e}").then(callbacks[0]).catch(callbacks[1])`)([n,r])}catch(e){r(e)}})}async function c({name:e,globalName:t,entry:n,createScriptHook:i}){let{entryExports:a}=r.getRemoteEntryExports(e,t);return a?a:"undefined"==typeof document?o.loadScriptNode(n,{attrs:{name:e,globalName:t},createScriptHook:i}).then(()=>{let{remoteEntryKey:o,entryExports:i}=r.getRemoteEntryExports(e,t);return r.assert(i,`
        Unable to use the ${e}'s '${n}' URL with ${o}'s globalName to get remoteEntry exports.
        Possible reasons could be:

        1. '${n}' is not the correct URL, or the remoteEntry resource or name is incorrect.

        2. ${o} cannot be used to get remoteEntry exports in the window object.
      `),i}).catch(e=>e):o.loadScript(n,{attrs:{},createScriptHook:i}).then(()=>{let{remoteEntryKey:o,entryExports:i}=r.getRemoteEntryExports(e,t);return r.assert(i,`
      Unable to use the ${e}'s '${n}' URL with ${o}'s globalName to get remoteEntry exports.
      Possible reasons could be:

      1. '${n}' is not the correct URL, or the remoteEntry resource or name is incorrect.

      2. ${o} cannot be used to get remoteEntry exports in the window object.
    `),i}).catch(e=>e)}function f(e){let{entry:t,name:n}=e;return o.composeKeyWithSeparator(n,t)}async function h({remoteEntryExports:e,remoteInfo:t,createScriptHook:n}){let{entry:o,name:i,type:a,entryGlobalName:s}=t,l=f(t);return e?e:(!r.globalLoading[l]&&("esm"===a?r.globalLoading[l]=u({entry:o,remoteEntryExports:e}):r.globalLoading[l]=c({name:i,globalName:s,entry:o,createScriptHook:n})),r.globalLoading[l])}function p(e){return l({},e,{entry:"entry"in e?e.entry:"",type:e.type||r.DEFAULT_REMOTE_TYPE,entryGlobalName:e.entryGlobalName||e.name,shareScope:e.shareScope||r.DEFAULT_SCOPE})}function d(){return(d=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e}).apply(this,arguments)}let m=class{async getEntry(){if(this.remoteEntryExports)return this.remoteEntryExports;let e=await h({remoteInfo:this.remoteInfo,remoteEntryExports:this.remoteEntryExports,createScriptHook:e=>{let t=this.host.loaderHook.lifecycle.createScript.emit({url:e});if("undefined"==typeof document||t instanceof HTMLScriptElement)return t}});return r.assert(e,`remoteEntryExports is undefined 
 ${r.safeToString(this.remoteInfo)}`),this.remoteEntryExports=e,this.remoteEntryExports}async get(e,t){let{loadFactory:n=!0}=t||{loadFactory:!0},o=await this.getEntry();if(!this.inited){let e=this.host.shareScopeMap,t=this.remoteInfo.shareScope||"default";!e[t]&&(e[t]={});let n=e[t],r=[],i={version:this.remoteInfo.version||""};Object.defineProperty(i,"hostId",{value:this.host.options.id||this.host.name,enumerable:!1});let a=await this.host.hooks.lifecycle.beforeInitContainer.emit({shareScope:n,remoteEntryInitOptions:i,initScope:r,remoteInfo:this.remoteInfo,origin:this.host});await o.init(a.shareScope,a.initScope,a.remoteEntryInitOptions),await this.host.hooks.lifecycle.initContainer.emit(d({},a,{remoteEntryExports:o}))}this.lib=o,this.inited=!0;let i=await o.get(e);return(r.assert(i,`${r.getFMId(this.remoteInfo)} remote don't export ${e}.`),n)?await i():i}constructor({remoteInfo:e,host:t}){this.inited=!1,this.lib=void 0,this.remoteInfo=e,this.host=t}};class y{on(e){"function"==typeof e&&this.listeners.add(e)}once(e){let t=this;this.on(function n(...r){return t.remove(n),e.apply(null,r)})}emit(...e){let t;return this.listeners.size>0&&this.listeners.forEach(n=>{t=n(...e)}),t}remove(e){this.listeners.delete(e)}removeAll(){this.listeners.clear()}constructor(e){this.type="",this.listeners=new Set,e&&(this.type=e)}}class b extends y{emit(...e){let t;let n=Array.from(this.listeners);if(n.length>0){let r=0,o=t=>!1!==t&&(r<n.length?Promise.resolve(n[r++].apply(null,e)).then(o):t);t=o()}return Promise.resolve(t)}}function g(e,t){if(!r.isObject(t))return!1;if(e!==t){for(let n in e)if(!(n in t))return!1}return!0}class v extends y{emit(e){for(let t of(!r.isObject(e)&&r.error(`The data for the "${this.type}" hook should be an object.`),this.listeners))try{let n=t(e);if(g(e,n))e=n;else{this.onerror(`A plugin returned an unacceptable value for the "${this.type}" type.`);break}}catch(e){r.warn(e),this.onerror(e)}return e}constructor(e){super(),this.onerror=r.error,this.type=e}}class _ extends y{emit(e){!r.isObject(e)&&r.error(`The response data for the "${this.type}" hook must be an object.`);let t=Array.from(this.listeners);if(t.length>0){let n=0,o=t=>(r.warn(t),this.onerror(t),e),i=r=>{if(g(e,r)){if(e=r,n<t.length)try{return Promise.resolve(t[n++](e)).then(i,o)}catch(e){return o(e)}}else this.onerror(`A plugin returned an incorrect value for the "${this.type}" type.`);return e};return Promise.resolve(i(e))}return Promise.resolve(e)}constructor(e){super(),this.onerror=r.error,this.type=e}}class E{applyPlugin(e){r.assert(r.isPlainObject(e),"Plugin configuration is invalid.");let t=e.name;r.assert(t,"A name must be provided by the plugin."),!this.registerPlugins[t]&&(this.registerPlugins[t]=e,Object.keys(this.lifecycle).forEach(t=>{let n=e[t];n&&this.lifecycle[t].on(n)}))}removePlugin(e){r.assert(e,"A name is required.");let t=this.registerPlugins[e];r.assert(t,`The plugin "${e}" is not registered.`),Object.keys(t).forEach(e=>{"name"!==e&&this.lifecycle[e].remove(t[e])})}inherit({lifecycle:e,registerPlugins:t}){Object.keys(e).forEach(t=>{r.assert(!this.lifecycle[t],`The hook "${t}" has a conflict and cannot be inherited.`),this.lifecycle[t]=e[t]}),Object.keys(t).forEach(e=>{r.assert(!this.registerPlugins[e],`The plugin "${e}" has a conflict and cannot be inherited.`),this.applyPlugin(t[e])})}constructor(e){this.registerPlugins={},this.lifecycle=e,this.lifecycleKeys=Object.keys(e)}}function S(){return(S=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e}).apply(this,arguments)}function O(e){return S({resourceCategory:"sync",share:!0,depsRemote:!0,prefetchInterface:!1},e)}function I(e,t){return t.map(t=>{let n=a(e,t.nameOrAlias);return r.assert(n,`Unable to preload ${t.nameOrAlias} as it is not included in ${!n&&r.safeToString({remoteInfo:n,remotes:e})}`),{remote:n,preloadConfig:O(t)}})}function w(e){return e?e.map(e=>"."===e?e:e.startsWith("./")?e.replace("./",""):e):[]}function $(e,t,n){let{cssAssets:r,jsAssetsWithoutEntry:i,entryAssets:a}=n;if(t.options.inBrowser){a.forEach(n=>{let{moduleInfo:r}=n,o=t.moduleCache.get(e.name);o?h({remoteInfo:r,remoteEntryExports:o.remoteEntryExports,createScriptHook:e=>{let n=t.loaderHook.lifecycle.createScript.emit({url:e});if(n instanceof HTMLScriptElement)return n}}):h({remoteInfo:r,remoteEntryExports:void 0,createScriptHook:e=>{let n=t.loaderHook.lifecycle.createScript.emit({url:e});if(n instanceof HTMLScriptElement)return n}})});let n=document.createDocumentFragment();r.forEach(e=>{let{link:r,needAttach:i}=o.createLink(e,()=>{},{rel:"preload",as:"style"},e=>{let n=t.loaderHook.lifecycle.createLink.emit({url:e});if(n instanceof HTMLLinkElement)return n});i&&n.appendChild(r)}),i.forEach(e=>{let{link:n,needAttach:r}=o.createLink(e,()=>{},{rel:"preload",as:"script"},e=>{let n=t.loaderHook.lifecycle.createLink.emit({url:e});if(n instanceof HTMLLinkElement)return n});r&&document.head.appendChild(n)}),document.head.appendChild(n)}}function T(){return(T=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e}).apply(this,arguments)}function R(e,t){(!("remoteEntry"in t)||!t.remoteEntry)&&r.error(`The attribute remoteEntry of ${name} must not be undefined.`);let{remoteEntry:n}=t,i=o.getResourceUrl(t,n);e.type=t.remoteEntryType,e.entryGlobalName=t.globalName,e.entry=i,e.version=t.version,e.buildVersion=t.buildVersion}function A(){return{name:"snapshot-plugin",async afterResolve(e){let{remote:t,pkgNameOrAlias:n,expose:o,origin:i,remoteInfo:a}=e;if(!r.isRemoteInfoWithEntry(t)||!r.isPureRemoteEntry(t)){let{remoteSnapshot:r,globalSnapshot:s}=await i.snapshotHandler.loadRemoteSnapshotInfo(t);R(a,r);let l={remote:t,preloadConfig:{nameOrAlias:n,exposes:[o],resourceCategory:"sync",share:!1,depsRemote:!1}},u=await i.hooks.lifecycle.generatePreloadAssets.emit({origin:i,preloadOptions:l,remoteInfo:a,remote:t,remoteSnapshot:r,globalSnapshot:s});return u&&$(a,i,u),T({},e,{remoteSnapshot:r})}return e}}}function N(e){let t=e.split(":");return 1===t.length?{name:t[0],version:void 0}:2===t.length?{name:t[0],version:t[1]}:{name:t[1],version:t[2]}}function P(e,t,n,i,a={},s){let l=r.getFMId(t),{value:u}=r.getInfoWithoutType(e,l),c=s||u;if(c&&!o.isManifestProvider(c)&&(n(c,t,i),c.remotesInfo))for(let t of Object.keys(c.remotesInfo)){if(a[t])continue;a[t]=!0;let r=N(t),o=c.remotesInfo[t];P(e,{name:r.name,version:o.matchedVersion},n,!1,a,void 0)}}function M(e,t,n,i,a){let s=[],l=[],u=[],c=new Set,f=new Set,{options:h}=e,{preloadConfig:p}=t,{depsRemote:d}=p;if(P(i,n,(t,n,i)=>{let a;if(i)a=p;else if(Array.isArray(d)){let e=d.find(e=>e.nameOrAlias===n.name||e.nameOrAlias===n.alias||!1);if(!e)return;a=O(e)}else{if(!0!==d)return;a=p}let c=o.getResourceUrl(t,"remoteEntry"in t?t.remoteEntry:"");c&&u.push({name:n.name,moduleInfo:{name:n.name,entry:c,type:"remoteEntryType"in t?t.remoteEntryType:"global",entryGlobalName:"globalName"in t?t.globalName:n.name,shareScope:"",version:"version"in t?t.version:void 0},url:c});let f="modules"in t?t.modules:[],h=w(a.exposes);if(h.length&&"modules"in t){var m;f=null==t?void 0:null==(m=t.modules)?void 0:m.reduce((e,t)=>((null==h?void 0:h.indexOf(t.moduleName))!==-1&&e.push(t),e),[])}function y(e){let n=e.map(e=>o.getResourceUrl(t,e));return a.filter?n.filter(a.filter):n}if(f){let o=f.length;for(let i=0;i<o;i++){let o=f[i],u=`${n.name}/${o.moduleName}`;if(e.hooks.lifecycle.handlePreloadModule.emit({id:"."===o.moduleName?n.name:u,name:n.name,remoteSnapshot:t,preloadConfig:a,remote:n,origin:e}),!r.getPreloaded(u))"all"===a.resourceCategory?(s.push(...y(o.assets.css.async)),s.push(...y(o.assets.css.sync)),l.push(...y(o.assets.js.async)),l.push(...y(o.assets.js.sync))):(a.resourceCategory="sync",s.push(...y(o.assets.css.sync)),l.push(...y(o.assets.js.sync))),r.setPreloaded(u)}}},!0,{},a),a.shared){let t=(t,n)=>{let o=r.getRegisteredShare(e.shareScopeMap,n.sharedName,t,e.hooks.lifecycle.resolveShare);o&&"function"==typeof o.lib&&(n.assets.js.sync.forEach(e=>{c.add(e)}),n.assets.css.sync.forEach(e=>{f.add(e)}))};a.shared.forEach(e=>{var n;let o=null==(n=h.shared)?void 0:n[e.sharedName];if(!o)return;let i=e.version?o.find(t=>t.version===e.version):o;if(!!i)r.arrayOptions(i).forEach(n=>{t(n,e)})})}let m=l.filter(e=>!c.has(e));return{cssAssets:s.filter(e=>!f.has(e)),jsAssetsWithoutEntry:m,entryAssets:u}}let j=function(){return{name:"generate-preload-assets-plugin",async generatePreloadAssets(e){let{origin:t,preloadOptions:n,remoteInfo:o,remote:i,globalSnapshot:a,remoteSnapshot:s}=e;return r.isRemoteInfoWithEntry(i)&&r.isPureRemoteEntry(i)?{cssAssets:[],jsAssetsWithoutEntry:[],entryAssets:[{name:i.name,url:i.entry,moduleInfo:{name:o.name,entry:i.entry,type:"global",entryGlobalName:"",shareScope:""}}]}:(R(o,s),M(t,n,o,a,s))}}};function k(){return(k=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e}).apply(this,arguments)}class D{async loadSnapshot(e){let{options:t}=this.HostInstance,{hostGlobalSnapshot:n,remoteSnapshot:r,globalSnapshot:o}=this.getGlobalRemoteInfo(e),{remoteSnapshot:i,globalSnapshot:a}=await this.hooks.lifecycle.loadSnapshot.emit({options:t,moduleInfo:e,hostGlobalSnapshot:n,remoteSnapshot:r,globalSnapshot:o});return{remoteSnapshot:i,globalSnapshot:a}}async loadRemoteSnapshotInfo(e){let{options:t}=this.HostInstance;await this.hooks.lifecycle.beforeLoadRemoteSnapshot.emit({options:t,moduleInfo:e});let n=r.getGlobalSnapshotInfoByModuleInfo({name:this.HostInstance.options.name,version:this.HostInstance.options.version});!n&&(n={version:this.HostInstance.options.version||"",remoteEntry:"",remotesInfo:{}},r.addGlobalSnapshot({[this.HostInstance.options.name]:n})),n&&"remotesInfo"in n&&!r.getInfoWithoutType(n.remotesInfo,e.name).value&&("version"in e||"entry"in e)&&(n.remotesInfo=k({},null==n?void 0:n.remotesInfo,{[e.name]:{matchedVersion:"version"in e?e.version:e.entry}}));let{hostGlobalSnapshot:i,remoteSnapshot:a,globalSnapshot:s}=this.getGlobalRemoteInfo(e),{remoteSnapshot:l,globalSnapshot:u}=await this.hooks.lifecycle.loadSnapshot.emit({options:t,moduleInfo:e,hostGlobalSnapshot:i,remoteSnapshot:a,globalSnapshot:s});if(l){if(o.isManifestProvider(l)){let t=await this.getManifestJson(l.remoteEntry,e,{}),n=r.setGlobalSnapshotInfoByModuleInfo(k({},e,{entry:l.remoteEntry}),t);return{remoteSnapshot:t,globalSnapshot:n}}{let{remoteSnapshot:t}=await this.hooks.lifecycle.loadRemoteSnapshot.emit({options:this.HostInstance.options,moduleInfo:e,remoteSnapshot:l,from:"global"});return{remoteSnapshot:t,globalSnapshot:u}}}if(r.isRemoteInfoWithEntry(e)){let t=await this.getManifestJson(e.entry,e,{}),n=r.setGlobalSnapshotInfoByModuleInfo(e,t),{remoteSnapshot:o}=await this.hooks.lifecycle.loadRemoteSnapshot.emit({options:this.HostInstance.options,moduleInfo:e,remoteSnapshot:t,from:"global"});return{remoteSnapshot:o,globalSnapshot:n}}r.error(`
          Cannot get remoteSnapshot with the name: '${e.name}', version: '${e.version}' from __FEDERATION__.moduleInfo. The following reasons may be causing the problem:

          1. The Deploy platform did not deliver the correct data. You can use __FEDERATION__.moduleInfo to check the remoteInfo.

          2. The remote '${e.name}' version '${e.version}' is not released.

          The transformed module info: ${JSON.stringify(u)}
        `)}getGlobalRemoteInfo(e){let t=r.getGlobalSnapshotInfoByModuleInfo({name:this.HostInstance.options.name,version:this.HostInstance.options.version}),n=t&&"remotesInfo"in t&&t.remotesInfo&&r.getInfoWithoutType(t.remotesInfo,e.name).value;return n&&n.matchedVersion?{hostGlobalSnapshot:t,globalSnapshot:r.getGlobalSnapshot(),remoteSnapshot:r.getGlobalSnapshotInfoByModuleInfo({name:e.name,version:n.matchedVersion})}:{hostGlobalSnapshot:void 0,globalSnapshot:r.getGlobalSnapshot(),remoteSnapshot:r.getGlobalSnapshotInfoByModuleInfo({name:e.name,version:"version"in e?e.version:void 0})}}async getManifestJson(e,t,n){let i=async()=>{let n=this.manifestCache.get(e);if(n)return n;try{let t=await this.loaderHook.lifecycle.fetch.emit(e,{});return(!t||!(t instanceof Response))&&(t=await fetch(e,{})),n=await t.json(),r.assert(n.metaData&&n.exposes&&n.shared,`${e} is not a federation manifest`),this.manifestCache.set(e,n),n}catch(n){r.error(`Failed to get manifestJson for ${t.name}. The manifest URL is ${e}. Please ensure that the manifestUrl is accessible.
          
 Error message:
          
 ${n}`)}},a=async()=>{let n=await i(),r=o.generateSnapshotFromManifest(n,{version:e}),{remoteSnapshot:a}=await this.hooks.lifecycle.loadRemoteSnapshot.emit({options:this.HostInstance.options,moduleInfo:t,manifestJson:n,remoteSnapshot:r,manifestUrl:e,from:"manifest"});return a};return!this.manifestLoading[e]&&(this.manifestLoading[e]=a().then(e=>e)),this.manifestLoading[e]}constructor(e){this.loadingHostSnapshot=null,this.manifestCache=new Map,this.hooks=new E({beforeLoadRemoteSnapshot:new b("beforeLoadRemoteSnapshot"),loadSnapshot:new _("loadGlobalSnapshot"),loadRemoteSnapshot:new _("loadRemoteSnapshot")}),this.manifestLoading=r.Global.__FEDERATION__.__MANIFEST_LOADING__,this.HostInstance=e,this.loaderHook=e.loaderHook}}function L(){return(L=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e}).apply(this,arguments)}function x(e,t){if(null==e)return{};var n,r,o={},i=Object.keys(e);for(r=0;r<i.length;r++)n=i[r],!(t.indexOf(n)>=0)&&(o[n]=e[n]);return o}class F{_setGlobalShareScopeMap(){let e=r.getGlobalShareScope(),t=this.options.id||this.options.name;t&&!e[t]&&(e[t]=this.shareScopeMap)}initOptions(e){this.registerPlugins(e.plugins);let t=this.formatOptions(this.options,e);return this.options=t,t}async loadShare(e,t){let n=r.getTargetSharedOptions({pkgName:e,extraOptions:t,shareInfos:this.options.shared});(null==n?void 0:n.scope)&&await Promise.all(n.scope.map(async e=>{await Promise.all(this.initializeSharing(e,n.strategy))}));let{shareInfo:o}=await this.hooks.lifecycle.beforeLoadShare.emit({pkgName:e,shareInfo:n,shared:this.options.shared,origin:this});r.assert(o,`Cannot find ${e} Share in the ${this.options.name}. Please ensure that the ${e} Share parameters have been injected`);let i=r.getRegisteredShare(this.shareScopeMap,e,o,this.hooks.lifecycle.resolveShare),a=e=>{!e.useIn&&(e.useIn=[]),r.addUniqueItem(e.useIn,this.options.name)};if(i&&i.lib)return a(i),i.lib;if(i&&i.loading&&!i.loaded){let e=await i.loading;return i.loaded=!0,!i.lib&&(i.lib=e),a(i),e}if(i){let t=(async()=>{let t=await i.get();o.lib=t,o.loaded=!0,a(o);let n=r.getRegisteredShare(this.shareScopeMap,e,o,this.hooks.lifecycle.resolveShare);return n&&(n.lib=t,n.loaded=!0),t})();return this.setShared({pkgName:e,loaded:!1,shared:i,from:this.options.name,lib:null,loading:t}),t}else{if(null==t?void 0:t.customShareInfo)return!1;let n=(async()=>{let t=await o.get();o.lib=t,o.loaded=!0,a(o);let n=r.getRegisteredShare(this.shareScopeMap,e,o,this.hooks.lifecycle.resolveShare);return n&&(n.lib=t,n.loaded=!0),t})();return this.setShared({pkgName:e,loaded:!1,shared:o,from:this.options.name,lib:null,loading:n}),n}}loadShareSync(e,t){let n=r.getTargetSharedOptions({pkgName:e,extraOptions:t,shareInfos:this.options.shared});(null==n?void 0:n.scope)&&n.scope.forEach(e=>{this.initializeSharing(e,n.strategy)});let o=r.getRegisteredShare(this.shareScopeMap,e,n,this.hooks.lifecycle.resolveShare),i=e=>{!e.useIn&&(e.useIn=[]),r.addUniqueItem(e.useIn,this.options.name)};if(o){if("function"==typeof o.lib)return i(o),!o.loaded&&(o.loaded=!0,o.from===this.options.name&&(n.loaded=!0)),o.lib;if("function"==typeof o.get){let t=o.get();if(!(t instanceof Promise))return i(o),this.setShared({pkgName:e,loaded:!0,from:this.options.name,lib:t,shared:o}),t}}if(n.lib)return!n.loaded&&(n.loaded=!0),n.lib;if(n.get){let t=n.get();if(t instanceof Promise)throw Error(`
        The loadShareSync function was unable to load ${e}. The ${e} could not be found in ${this.options.name}.
        Possible reasons for failure: 

        1. The ${e} share was registered with the 'get' attribute, but loadShare was not used beforehand.

        2. The ${e} share was not registered with the 'lib' attribute.

      `);return n.lib=t,this.setShared({pkgName:e,loaded:!0,from:this.options.name,lib:n.lib,shared:n}),n.lib}throw Error(`
        The loadShareSync function was unable to load ${e}. The ${e} could not be found in ${this.options.name}.
        Possible reasons for failure: 

        1. The ${e} share was registered with the 'get' attribute, but loadShare was not used beforehand.

        2. The ${e} share was not registered with the 'lib' attribute.

      `)}initRawContainer(e,t,n){let r=new m({host:this,remoteInfo:p({name:e,entry:t})});return r.remoteEntryExports=n,this.moduleCache.set(e,r),r}async _getRemoteModuleAndOptions(e){let{id:t}=await this.hooks.lifecycle.beforeRequest.emit({id:e,options:this.options,origin:this}),n=i(this.options.remotes,t);r.assert(n,`
        Unable to locate ${t} in ${this.options.name}. Potential reasons for failure include:

        1. ${t} was not included in the 'remotes' parameter of ${this.options.name||"the host"}.

        2. ${t} could not be found in the 'remotes' of ${this.options.name} with either 'name' or 'alias' attributes.
        3. ${t} is not online, injected, or loaded.
        4. ${t}  cannot be accessed on the expected.
        5. The 'beforeRequest' hook was provided but did not return the correct 'remoteInfo' when attempting to load ${t}.
      `);let{remote:o}=n,a=p(o),s=await this.hooks.lifecycle.afterResolve.emit(L({id:t},n,{options:this.options,origin:this,remoteInfo:a})),{remote:l,expose:u}=s;r.assert(l&&u,`The 'beforeRequest' hook was executed, but it failed to return the correct 'remote' and 'expose' values while loading ${t}.`);let c=this.moduleCache.get(l.name),f={host:this,remoteInfo:a};return!c&&(c=new m(f),this.moduleCache.set(l.name,c)),{module:c,moduleOptions:f,remoteMatchInfo:s}}async loadRemote(e,t){try{let{loadFactory:n=!0}=t||{loadFactory:!0},{module:r,moduleOptions:o,remoteMatchInfo:i}=await this._getRemoteModuleAndOptions(e),{pkgNameOrAlias:a,remote:s,expose:l,id:u}=i,c=await r.get(l,t),f=await this.hooks.lifecycle.onLoad.emit({id:u,pkgNameOrAlias:a,expose:l,exposeModule:n?c:void 0,exposeModuleFactory:n?void 0:c,remote:s,options:o,moduleInstance:r,origin:this});if("function"==typeof f)return f;return c}catch(o){let{from:n="runtime"}=t||{from:"runtime"},r=await this.hooks.lifecycle.errorLoadRemote.emit({id:e,error:o,from:n,origin:this});if(!r)throw o;return r}}async preloadRemote(e){await this.hooks.lifecycle.beforePreloadRemote.emit({preloadOptions:e,options:this.options,origin:this});let t=I(this.options.remotes,e);await Promise.all(t.map(async e=>{let{remote:t}=e,n=p(t),{globalSnapshot:r,remoteSnapshot:o}=await this.snapshotHandler.loadRemoteSnapshotInfo(t),i=await this.hooks.lifecycle.generatePreloadAssets.emit({origin:this,preloadOptions:e,remote:t,remoteInfo:n,globalSnapshot:r,remoteSnapshot:o});if(!!i)$(n,this,i)}))}initializeSharing(e=r.DEFAULT_SCOPE,t){let n=this.shareScopeMap,o=this.options.name;!n[e]&&(n[e]={});let i=n[e],a=(e,t)=>{var n;let{version:r,eager:a}=t;i[e]=i[e]||{};let s=i[e],l=s[r],u=!!(l&&(l.eager||(null==(n=l.shareConfig)?void 0:n.eager)));(!l||"loaded-first"!==l.strategy&&!l.loaded&&(!a!=!u?a:o>l.from))&&(s[r]=t)},s=[],l=t=>t&&t.init&&t.init(n[e]),u=async e=>{let{module:t}=await this._getRemoteModuleAndOptions(e);if(t.getEntry){let e=await t.getEntry();!t.inited&&(l(e),t.inited=!0)}};return Object.keys(this.options.shared).forEach(t=>{this.options.shared[t].forEach(n=>{n.scope.includes(e)&&a(t,n)})}),"version-first"===t&&this.options.remotes.forEach(t=>{t.shareScope===e&&s.push(u(t.name))}),s}initShareScopeMap(e,t){this.shareScopeMap[e]=t,this.hooks.lifecycle.initContainerShareScopeMap.emit({shareScope:t,options:this.options,origin:this})}formatOptions(e,t){let n=r.formatShareConfigs(t.shared||{},t.name),o=L({},e.shared);Object.keys(n).forEach(e=>{o[e]?n[e].forEach(t=>{!o[e].find(e=>e.version===t.version)&&o[e].push(t)}):o[e]=n[e]});let{userOptions:i,options:a}=this.hooks.lifecycle.beforeInit.emit({origin:this,userOptions:t,options:e,shareInfo:o}),s=(i.remotes||[]).reduce((e,t)=>(this.registerRemote(t,e,{force:!1}),e),a.remotes);Object.keys(n).forEach(e=>{n[e].forEach(n=>{!r.getRegisteredShare(this.shareScopeMap,e,n,this.hooks.lifecycle.resolveShare)&&n&&n.lib&&this.setShared({pkgName:e,lib:n.lib,get:n.get,loaded:!0,shared:n,from:t.name})})});let l=[...a.plugins];i.plugins&&i.plugins.forEach(e=>{!l.includes(e)&&l.push(e)});let u=L({},e,t,{plugins:l,remotes:s,shared:o});return this.hooks.lifecycle.init.emit({origin:this,options:u}),u}registerPlugins(e){let t=s(e,[this.hooks,this.snapshotHandler.hooks,this.loaderHook]);this.options.plugins=this.options.plugins.reduce((e,t)=>(e&&!e.find(e=>e.name===t.name)&&e.push(t),e),t||[])}setShared({pkgName:e,shared:t,from:n,lib:r,loading:o,loaded:i,get:a}){let{version:s,scope:l="default"}=t,u=x(t,["version","scope"]);(Array.isArray(l)?l:[l]).forEach(t=>{if(!this.shareScopeMap[t]&&(this.shareScopeMap[t]={}),!this.shareScopeMap[t][e]&&(this.shareScopeMap[t][e]={}),!this.shareScopeMap[t][e][s])this.shareScopeMap[t][e][s]=L({version:s,scope:["default"]},u,{lib:r,loaded:i,loading:o}),a&&(this.shareScopeMap[t][e][s].get=a)})}removeRemote(e){let{name:t}=e,n=this.options.remotes.findIndex(e=>e.name===t);-1!==n&&this.options.remotes.splice(n,1);let o=this.moduleCache.get(e.name);if(o){let t=o.remoteInfo.entryGlobalName;globalThis[t]&&delete globalThis[t];let n=f(o.remoteInfo);r.globalLoading[n]&&delete r.globalLoading[n],this.moduleCache.delete(e.name)}}registerRemote(e,t,n){let o=()=>{if(e.alias){let n=t.find(t=>{var n;return e.alias&&(t.name.startsWith(e.alias)||(null==(n=t.alias)?void 0:n.startsWith(e.alias)))});r.assert(!n,`The alias ${e.alias} of remote ${e.name} is not allowed to be the prefix of ${n&&n.name} name or alias`)}"entry"in e&&r.isBrowserEnv()&&!e.entry.startsWith("http")&&(e.entry=new URL(e.entry,window.location.origin).href),!e.shareScope&&(e.shareScope=r.DEFAULT_SCOPE),!e.type&&(e.type=r.DEFAULT_REMOTE_TYPE)},i=t.find(t=>t.name===e.name);if(i){let a=[`The remote "${e.name}" is already registered.`,(null==n?void 0:n.force)?"Hope you have known that OVERRIDE it may have some unexpected errors":'If you want to merge the remote, you can set "force: true".'];(null==n?void 0:n.force)&&(this.removeRemote(i),o(),t.push(e)),r.warn(a.join(" "))}else o(),t.push(e)}registerRemotes(e,t){e.forEach(e=>{this.registerRemote(e,this.options.remotes,{force:null==t?void 0:t.force})})}constructor(e){this.hooks=new E({beforeInit:new v("beforeInit"),init:new y,beforeRequest:new _("beforeRequest"),afterResolve:new _("afterResolve"),beforeInitContainer:new _("beforeInitContainer"),initContainerShareScopeMap:new _("initContainer"),initContainer:new _("initContainer"),onLoad:new b("onLoad"),handlePreloadModule:new y("handlePreloadModule"),errorLoadRemote:new b("errorLoadRemote"),beforeLoadShare:new _("beforeLoadShare"),loadShare:new b,resolveShare:new v("resolveShare"),beforePreloadRemote:new b,generatePreloadAssets:new b("generatePreloadAssets"),afterPreloadRemote:new b}),this.version="0.1.6",this.moduleCache=new Map,this.loaderHook=new E({getModuleInfo:new y,createScript:new y,createLink:new y,fetch:new b("fetch")});let t={id:r.getBuilderId(),name:e.name,plugins:[A(),j()],remotes:[],shared:{},inBrowser:r.isBrowserEnv()};this.name=e.name,this.options=t,this.shareScopeMap={},this._setGlobalShareScopeMap(),this.snapshotHandler=new D(this),this.registerPlugins([...t.plugins,...e.plugins||[]]),this.options=this.formatOptions(t,e)}}let C=null;function H(e){let t=r.getGlobalFederationInstance(e.name,e.version);return t?(t.initOptions(e),!C&&(C=t),t):(C=new(r.getGlobalFederationConstructor()||F)(e),r.setGlobalFederationInstance(C),C)}function G(...e){return r.assert(C,"Please call init first"),C.loadRemote.apply(C,e)}function U(...e){return r.assert(C,"Please call init first"),C.loadShare.apply(C,e)}function B(...e){return r.assert(C,"Please call init first"),C.loadShareSync.apply(C,e)}function q(...e){return r.assert(C,"Please call init first"),C.preloadRemote.apply(C,e)}function W(...e){return r.assert(C,"Please call init first"),C.registerRemotes.apply(C,e)}function V(...e){return r.assert(C,"Please call init first"),C.registerPlugins.apply(C,e)}r.setGlobalFederationConstructor(F),t.registerGlobalPlugins=r.registerGlobalPlugins,Object.defineProperty(t,"loadScript",{enumerable:!0,get:function(){return o.loadScript}}),Object.defineProperty(t,"loadScriptNode",{enumerable:!0,get:function(){return o.loadScriptNode}}),t.FederationHost=F,t.getRemoteEntry=h,t.getRemoteInfo=p,t.init=H,t.loadRemote=G,t.loadShare=U,t.loadShareSync=B,t.preloadRemote=q,t.registerPlugins=V,t.registerRemotes=W},27303:function(e,t){"use strict";function n(){return"undefined"!=typeof FEDERATION_BUILD_IDENTIFIER?FEDERATION_BUILD_IDENTIFIER:""}function r(){return false}function o(){return"undefined"!=typeof window}let i="[ Federation Runtime ]";function a(e,t){!e&&s(t)}function s(e){if(e instanceof Error)throw e.message=`${i}: ${e.message}`,e;throw Error(`${i}: ${e}`)}function l(e){e instanceof Error?(e.message=`${i}: ${e.message}`,console.warn(e)):console.warn(`${i}: ${e}`)}function u(e,t){return -1===e.findIndex(e=>e===t)&&e.push(t),e}function c(e){return"version"in e&&e.version?`${e.name}:${e.version}`:"entry"in e&&e.entry?`${e.name}:${e.entry}`:`${e.name}`}function f(e){return void 0!==e.entry}function h(e){return!e.entry.includes(".json")&&e.entry.includes(".js")}function p(e){try{return JSON.stringify(e,null,2)}catch(e){return""}}function d(e){return e&&"object"==typeof e}let m=Object.prototype.toString;function y(e){return"[object Object]"===m.call(e)}function b(e){return Array.isArray(e)?e:[e]}function g(){return(g=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e}).apply(this,arguments)}function v(e,t){if(null==e)return{};var n,r,o={},i=Object.keys(e);for(r=0;r<i.length;r++)n=i[r],!(t.indexOf(n)>=0)&&(o[n]=e[n]);return o}let _=(()=>{try{return Function("return this")()}catch(e){return globalThis}})(),E=_;function S(e,t,n){Object.defineProperty(e,t,{value:n,configurable:!1,writable:!0})}function O(e,t){return Object.hasOwnProperty.call(e,t)}!O(globalThis,"__GLOBAL_LOADING_REMOTE_ENTRY__")&&S(globalThis,"__GLOBAL_LOADING_REMOTE_ENTRY__",{});let I=globalThis.__GLOBAL_LOADING_REMOTE_ENTRY__;function w(e){var t,n,r,o,i,a,s,l,u,c,f,h;O(e,"__VMOK__")&&!O(e,"__FEDERATION__")&&S(e,"__FEDERATION__",e.__VMOK__),!O(e,"__FEDERATION__")&&(S(e,"__FEDERATION__",{__GLOBAL_PLUGIN__:[],__INSTANCES__:[],moduleInfo:{},__SHARE__:{},__MANIFEST_LOADING__:{},__PRELOADED_MAP__:new Map}),S(e,"__VMOK__",e.__FEDERATION__)),null!=(s=(t=e.__FEDERATION__).__GLOBAL_PLUGIN__)||(t.__GLOBAL_PLUGIN__=[]),null!=(l=(n=e.__FEDERATION__).__INSTANCES__)||(n.__INSTANCES__=[]),null!=(u=(r=e.__FEDERATION__).moduleInfo)||(r.moduleInfo={}),null!=(c=(o=e.__FEDERATION__).__SHARE__)||(o.__SHARE__={}),null!=(f=(i=e.__FEDERATION__).__MANIFEST_LOADING__)||(i.__MANIFEST_LOADING__={}),null!=(h=(a=e.__FEDERATION__).__PRELOADED_MAP__)||(a.__PRELOADED_MAP__=new Map)}function $(){globalThis.__FEDERATION__.__GLOBAL_PLUGIN__=[],globalThis.__FEDERATION__.__INSTANCES__=[],globalThis.__FEDERATION__.moduleInfo={},globalThis.__FEDERATION__.__SHARE__={},globalThis.__FEDERATION__.__MANIFEST_LOADING__={}}function T(e,t){let r=n();return globalThis.__FEDERATION__.__INSTANCES__.find(o=>!!r&&o.options.id===n()||o.options.name===e&&!o.options.version&&!t||o.options.name===e&&!!t&&o.options.version===t||!1)}function R(e){globalThis.__FEDERATION__.__INSTANCES__.push(e)}function A(){return globalThis.__FEDERATION__.__DEBUG_CONSTRUCTOR__}function N(e,t=r()){t&&(globalThis.__FEDERATION__.__DEBUG_CONSTRUCTOR__=e,globalThis.__FEDERATION__.__DEBUG_CONSTRUCTOR_VERSION__="0.1.6")}function P(e,t){if("string"==typeof t){if(e[t])return{value:e[t],key:t};for(let n of Object.keys(e)){let[r,o]=n.split(":"),i=`${r}:${t}`,a=e[i];if(a)return{value:a,key:i}}return{value:void 0,key:t}}throw Error("key must be string")}w(globalThis),w(_);let M=()=>_.__FEDERATION__.moduleInfo,j=(e,t)=>{let n=P(t,c(e)).value;if(n&&!n.version&&"version"in e&&e.version&&(n.version=e.version),n)return n;if("version"in e&&e.version){let{version:t}=e,n=c(v(e,["version"])),r=P(_.__FEDERATION__.moduleInfo,n).value;if((null==r?void 0:r.version)===t)return r}},k=e=>j(e,_.__FEDERATION__.moduleInfo),D=(e,t)=>{let n=c(e);return _.__FEDERATION__.moduleInfo[n]=t,_.__FEDERATION__.moduleInfo},L=e=>(_.__FEDERATION__.moduleInfo=g({},_.__FEDERATION__.moduleInfo,e),()=>{for(let t of Object.keys(e))delete _.__FEDERATION__.moduleInfo[t]}),x=(e,t)=>{let n=t||`__FEDERATION_${e}:custom__`,r=globalThis[n];return{remoteEntryKey:n,entryExports:r}},F=e=>{let{__GLOBAL_PLUGIN__:t}=_.__FEDERATION__;e.forEach(e=>{-1===t.findIndex(t=>t.name===e.name)?t.push(e):l(`The plugin ${e.name} has been registered.`)})},C=()=>_.__FEDERATION__.__GLOBAL_PLUGIN__,H=e=>globalThis.__FEDERATION__.__PRELOADED_MAP__.get(e),G=e=>globalThis.__FEDERATION__.__PRELOADED_MAP__.set(e,!0),U="default",B="global",q="[0-9A-Za-z-]+",W=`(?:\\+(${q}(?:\\.${q})*))`,V="0|[1-9]\\d*",z="[0-9]+",Y="\\d*[a-zA-Z-][a-zA-Z0-9-]*",K=`(?:${z}|${Y})`,J=`(?:-?(${K}(?:\\.${K})*))`,X=`(?:${V}|${Y})`,Z=`(?:-(${X}(?:\\.${X})*))`,Q=`${V}|x|X|\\*`,ee=`[v=\\s]*(${Q})(?:\\.(${Q})(?:\\.(${Q})(?:${Z})?${W}?)?)?`,et=`^\\s*(${ee})\\s+-\\s+(${ee})\\s*$`,en=`(${z})\\.(${z})\\.(${z})`,er=`[v=\\s]*${en}${J}?${W}?`,eo="((?:<|>)?=?)",ei=`(\\s*)${eo}\\s*(${er}|${ee})`,ea="(?:~>?)",es=`(\\s*)${ea}\\s+`,el="(?:\\^)",eu=`(\\s*)${el}\\s+`,ec="(<|>)?=?\\s*\\*",ef=`^${el}${ee}$`,eh=`(${V})\\.(${V})\\.(${V})`,ep=`v?${eh}${Z}?${W}?`,ed=`^${ea}${ee}$`,em=`^${eo}\\s*${ee}$`,ey=`^${eo}\\s*(${ep})$|^$`,eb="^\\s*>=\\s*0.0.0\\s*$";function eg(e){return new RegExp(e)}function ev(e){return!e||"x"===e.toLowerCase()||"*"===e}function e_(...e){return t=>e.reduce((e,t)=>t(e),t)}function eE(e){return e.match(eg(ey))}function eS(e,t,n,r){let o=`${e}.${t}.${n}`;return r?`${o}-${r}`:o}function eO(e){return e.replace(eg(et),(e,t,n,r,o,i,a,s,l,u,c,f)=>(t=ev(n)?"":ev(r)?`>=${n}.0.0`:ev(o)?`>=${n}.${r}.0`:`>=${t}`,s=ev(l)?"":ev(u)?`<${Number(l)+1}.0.0-0`:ev(c)?`<${l}.${Number(u)+1}.0-0`:f?`<=${l}.${u}.${c}-${f}`:`<=${s}`,`${t} ${s}`.trim()))}function eI(e){return e.replace(eg(ei),"$1$2$3")}function ew(e){return e.replace(eg(es),"$1~")}function e$(e){return e.replace(eg(eu),"$1^")}function eT(e){return e.trim().split(/\s+/).map(e=>e.replace(eg(ef),(e,t,n,r,o)=>{if(ev(t))return"";if(ev(n))return`>=${t}.0.0 <${Number(t)+1}.0.0-0`;if(ev(r))return"0"===t?`>=${t}.${n}.0 <${t}.${Number(n)+1}.0-0`:`>=${t}.${n}.0 <${Number(t)+1}.0.0-0`;else{if(o)return"0"!==t?`>=${t}.${n}.${r}-${o} <${Number(t)+1}.0.0-0`:"0"===n?`>=${t}.${n}.${r}-${o} <${t}.${n}.${Number(r)+1}-0`:`>=${t}.${n}.${r}-${o} <${t}.${Number(n)+1}.0-0`;if("0"===t)return"0"===n?`>=${t}.${n}.${r} <${t}.${n}.${Number(r)+1}-0`:`>=${t}.${n}.${r} <${t}.${Number(n)+1}.0-0`;return`>=${t}.${n}.${r} <${Number(t)+1}.0.0-0`}})).join(" ")}function eR(e){return e.trim().split(/\s+/).map(e=>e.replace(eg(ed),(e,t,n,r,o)=>{if(ev(t))return"";if(ev(n))return`>=${t}.0.0 <${Number(t)+1}.0.0-0`;if(ev(r))return`>=${t}.${n}.0 <${t}.${Number(n)+1}.0-0`;else if(o)return`>=${t}.${n}.${r}-${o} <${t}.${Number(n)+1}.0-0`;return`>=${t}.${n}.${r} <${t}.${Number(n)+1}.0-0`})).join(" ")}function eA(e){return e.split(/\s+/).map(e=>e.trim().replace(eg(em),(e,t,n,r,o,i)=>{let a=ev(n),s=a||ev(r),l=s||ev(o);if("="===t&&l&&(t=""),i="",a)return">"===t||"<"===t?"<0.0.0-0":"*";if(t&&l)return s&&(r=0),o=0,">"===t?(t=">=",s?(n=Number(n)+1,r=0):r=Number(r)+1,o=0):"<="===t&&(t="<",s?n=Number(n)+1:r=Number(r)+1),"<"===t&&(i="-0"),`${t+n}.${r}.${o}${i}`;if(s)return`>=${n}.0.0${i} <${Number(n)+1}.0.0-0`;else if(l)return`>=${n}.${r}.0${i} <${n}.${Number(r)+1}.0-0`;return e})).join(" ")}function eN(e){return e.trim().replace(eg(ec),"")}function eP(e){return e.trim().replace(eg(eb),"")}function eM(e,t){return(e=Number(e)||e,e>(t=Number(t)||t))?1:e===t?0:-1}function ej(e,t){let{preRelease:n}=e,{preRelease:r}=t;if(void 0===n&&r)return 1;if(n&&void 0===r)return -1;if(void 0===n&&void 0===r)return 0;for(let e=0,t=n.length;e<=t;e++){let t=n[e],o=r[e];if(t!==o){if(void 0===t&&void 0===o)break;if(!t)return 1;if(!o)return -1;return eM(t,o)}}return 0}function ek(e,t){return eM(e.major,t.major)||eM(e.minor,t.minor)||eM(e.patch,t.patch)||ej(e,t)}function eD(e,t){return e.version===t.version}function eL(e,t){switch(e.operator){case"":case"=":return eD(e,t);case">":return 0>ek(e,t);case">=":return eD(e,t)||0>ek(e,t);case"<":return ek(e,t)>0;case"<=":return eD(e,t)||ek(e,t)>0;case void 0:return!0;default:return!1}}function ex(e){return e_(eT,eR,eA,eN)(e)}function eF(e){return e_(eO,eI,ew,e$)(e.trim()).split(/\s+/).join(" ")}function eC(e,t){if(!e)return!1;let n=eF(t).split(" ").map(e=>ex(e)).join(" ").split(/\s+/).map(e=>eP(e)),r=eE(e);if(!r)return!1;let[,o,,i,a,s,l]=r,u={operator:o,version:eS(i,a,s,l),major:i,minor:a,patch:s,preRelease:null==l?void 0:l.split(".")};for(let e of n){let t=eE(e);if(!t)return!1;let[,n,,r,o,i,a]=t;if(!eL({operator:n,version:eS(r,o,i,a),major:r,minor:o,patch:i,preRelease:null==a?void 0:a.split(".")},u))return!1}return!0}function eH(){return(eH=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)Object.prototype.hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e}).apply(this,arguments)}function eG(e,t,n){var r;let o;return o="get"in e?e.get:"lib"in e?()=>Promise.resolve(e.lib):()=>Promise.resolve(()=>{throw Error(`Can not get shared '${n}'!`)}),eH({deps:[],useIn:[],from:t,loading:null},e,{shareConfig:eH({requiredVersion:`^${e.version}`,singleton:!1,eager:!1,strictVersion:!1},e.shareConfig),get:o,loaded:"lib"in e||void 0,version:null!=(r=e.version)?r:"0",scope:Array.isArray(e.scope)?e.scope:["default"],strategy:e.strategy||"version-first"})}function eU(e,t){return e?Object.keys(e).reduce((n,r)=>{let o=b(e[r]);return n[r]=n[r]||[],o.forEach(e=>{n[r].push(eG(e,t,r))}),n},{}):{}}function eB(e,t){let n=e=>{if(!Number.isNaN(Number(e))){let t=e.split("."),n=e;for(let e=0;e<3-t.length;e++)n+=".0";return n}return e};return!!eC(n(e),`<=${n(t)}`)||!1}let eq=(e,t)=>{let n=t||function(e,t){return eB(e,t)};return Object.keys(e).reduce((e,t)=>!e||n(e,t)||"0"===e?t:e,0)},eW=e=>!!e.loaded||"function"==typeof e.lib;function eV(e,t,n){let r=e[t][n],o=function(e,t){return!eW(r[e])&&eB(e,t)};return eq(e[t][n],o)}function ez(e,t,n){let r=e[t][n],o=function(e,t){if(eW(r[t]))return!eW(r[e])||!!eB(e,t);return!eW(r[e])&&eB(e,t)};return eq(e[t][n],o)}function eY(e){return"loaded-first"===e?ez:eV}function eK(e,t,n,r){if(!e)return;let{shareConfig:o,scope:i=U,strategy:a}=n;for(let u of Array.isArray(i)?i:[i])if(o&&e[u]&&e[u][t]){let{requiredVersion:i}=o,c=eY(a)(e,u,t),f=()=>{if(o.singleton){if("string"==typeof i&&!eC(c,i)){let r=`Version ${c} from ${c&&e[u][t][c].from} of shared singleton module ${t} does not satisfy the requirement of ${n.from} which needs ${i})`;o.strictVersion?s(r):l(r)}return e[u][t][c]}if(!1===i||"*"===i||eC(c,i))return e[u][t][c];for(let[n,r]of Object.entries(e[u][t]))if(eC(n,i))return r},h={shareScopeMap:e,scope:u,pkgName:t,version:c,GlobalFederation:E.__FEDERATION__,resolver:f};return(r.emit(h)||h).resolver()}}function eJ(){return E.__FEDERATION__.__SHARE__}function eX(e){var t;let{pkgName:n,extraOptions:r,shareInfos:o}=e,i=e=>{if(!e)return;let t={};e.forEach(e=>{t[e.version]=e});let n=function(e,n){return!eW(t[e])&&eB(e,n)},r=eq(t,n);return t[r]};return Object.assign({},(null!=(t=null==r?void 0:r.resolver)?t:i)(o[n]),null==r?void 0:r.customShareInfo)}t.DEFAULT_REMOTE_TYPE=B,t.DEFAULT_SCOPE=U,t.Global=E,t.addGlobalSnapshot=L,t.addUniqueItem=u,t.arrayOptions=b,t.assert=a,t.error=s,t.formatShareConfigs=eU,t.getBuilderId=n,t.getFMId=c,t.getGlobalFederationConstructor=A,t.getGlobalFederationInstance=T,t.getGlobalHostPlugins=C,t.getGlobalShareScope=eJ,t.getGlobalSnapshot=M,t.getGlobalSnapshotInfoByModuleInfo=k,t.getInfoWithoutType=P,t.getPreloaded=H,t.getRegisteredShare=eK,t.getRemoteEntryExports=x,t.getTargetSharedOptions=eX,t.getTargetSnapshotInfoByModuleInfo=j,t.globalLoading=I,t.isBrowserEnv=o,t.isObject=d,t.isPlainObject=y,t.isPureRemoteEntry=h,t.isRemoteInfoWithEntry=f,t.nativeGlobal=_,t.registerGlobalPlugins=F,t.resetFederationGlobalInfo=$,t.safeToString=p,t.setGlobalFederationConstructor=N,t.setGlobalFederationInstance=R,t.setGlobalSnapshotInfoByModuleInfo=D,t.setPreloaded=G,t.warn=l},5580:function(__unused_webpack_module,exports,__webpack_require__){"use strict";var _obj,_obj1,process=__webpack_require__(14224);function _define_property$3(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}Object.defineProperty(exports,"__esModule",{value:!0});var FederationModuleManifest="federation-manifest.json",MANIFEST_EXT=".json",BROWSER_LOG_KEY="FEDERATION_DEBUG",BROWSER_LOG_VALUE="1",NameTransformSymbol={AT:"@",HYPHEN:"-",SLASH:"/"},NameTransformMap=(_obj={},_define_property$3(_obj,NameTransformSymbol.AT,"scope_"),_define_property$3(_obj,NameTransformSymbol.HYPHEN,"_"),_define_property$3(_obj,NameTransformSymbol.SLASH,"__"),_obj),EncodedNameTransformMap=(_obj1={},_define_property$3(_obj1,NameTransformMap[NameTransformSymbol.AT],NameTransformSymbol.AT),_define_property$3(_obj1,NameTransformMap[NameTransformSymbol.HYPHEN],NameTransformSymbol.HYPHEN),_define_property$3(_obj1,NameTransformMap[NameTransformSymbol.SLASH],NameTransformSymbol.SLASH),_obj1),SEPARATOR=":",ManifestFileName="mf-manifest.json",StatsFileName="mf-stats.json",MFModuleType={NPM:"npm",APP:"app"},MODULE_DEVTOOL_IDENTIFIER="__MF_DEVTOOLS_MODULE_INFO__",ContainerPlugin=Object.freeze({__proto__:null}),ContainerReferencePlugin=Object.freeze({__proto__:null}),ModuleFederationPlugin=Object.freeze({__proto__:null}),SharePlugin=Object.freeze({__proto__:null});function isBrowserEnv(){return"undefined"!=typeof window}function isDebugMode(){return void 0!==process&&process.env&&process.env.FEDERATION_DEBUG?!!process.env.FEDERATION_DEBUG:"undefined"!=typeof FEDERATION_DEBUG&&!!FEDERATION_DEBUG}var getProcessEnv=function(){return void 0!==process&&process.env?process.env:{}};function _array_like_to_array$2(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=Array(t);n<t;n++)r[n]=e[n];return r}function _array_without_holes(e){if(Array.isArray(e))return _array_like_to_array$2(e)}function _class_call_check(e,t){if(!(e instanceof t))throw TypeError("Cannot call a class as a function")}function _defineProperties(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,r.key,r)}}function _create_class(e,t,n){return t&&_defineProperties(e.prototype,t),n&&_defineProperties(e,n),e}function _define_property$2(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function _iterable_to_array$1(e){if("undefined"!=typeof Symbol&&null!=e[Symbol.iterator]||null!=e["@@iterator"])return Array.from(e)}function _non_iterable_spread(){throw TypeError("Invalid attempt to spread non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function _to_consumable_array(e){return _array_without_holes(e)||_iterable_to_array$1(e)||_unsupported_iterable_to_array$2(e)||_non_iterable_spread()}function _unsupported_iterable_to_array$2(e,t){if(e){if("string"==typeof e)return _array_like_to_array$2(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if("Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n)return Array.from(n);if("Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return _array_like_to_array$2(e,t)}}function safeToString(e){try{return JSON.stringify(e,null,2)}catch(e){return""}}var DEBUG_LOG="[ FEDERATION DEBUG ]";function safeGetLocalStorageItem(){try{if("undefined"!=typeof window&&window.localStorage)return localStorage.getItem(BROWSER_LOG_KEY)===BROWSER_LOG_VALUE}catch(e){return"undefined"!=typeof document}return!1}var Logger=function(){function e(t){_class_call_check(this,e),_define_property$2(this,"enable",!1),_define_property$2(this,"identifier",void 0),this.identifier=t||DEBUG_LOG,isBrowserEnv()&&safeGetLocalStorageItem()?this.enable=!0:isDebugMode()&&(this.enable=!0)}return _create_class(e,[{key:"info",value:function(e,t){if(this.enable){var n=safeToString(t)||"";isBrowserEnv()?console.info("%c ".concat(this.identifier,": ").concat(e," ").concat(n),"color:#3300CC"):console.info("\x1b[34m%s","".concat(this.identifier,": ").concat(e," ").concat(n?"\n".concat(n):""))}}},{key:"logOriginalInfo",value:function(){for(var e,t,n=arguments.length,r=Array(n),o=0;o<n;o++)r[o]=arguments[o];this.enable&&(isBrowserEnv()?(console.info("%c ".concat(this.identifier,": OriginalInfo"),"color:#3300CC"),(e=console).log.apply(e,_to_consumable_array(r))):(console.info("%c ".concat(this.identifier,": OriginalInfo"),"color:#3300CC"),(t=console).log.apply(t,_to_consumable_array(r))))}}]),e}();function _array_like_to_array$1(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=Array(t);n<t;n++)r[n]=e[n];return r}function _array_with_holes$1(e){if(Array.isArray(e))return e}function _iterable_to_array(e){if("undefined"!=typeof Symbol&&null!=e[Symbol.iterator]||null!=e["@@iterator"])return Array.from(e)}function _iterable_to_array_limit$1(e,t){var n,r,o=null==e?null:"undefined"!=typeof Symbol&&e[Symbol.iterator]||e["@@iterator"];if(null!=o){var i=[],a=!0,s=!1;try{for(o=o.call(e);!(a=(n=o.next()).done)&&(i.push(n.value),!t||i.length!==t);a=!0);}catch(e){s=!0,r=e}finally{try{!a&&null!=o.return&&o.return()}finally{if(s)throw r}}return i}}function _non_iterable_rest$1(){throw TypeError("Invalid attempt to destructure non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function _sliced_to_array$1(e,t){return _array_with_holes$1(e)||_iterable_to_array_limit$1(e,t)||_unsupported_iterable_to_array$1(e,t)||_non_iterable_rest$1()}function _to_array(e){return _array_with_holes$1(e)||_iterable_to_array(e)||_unsupported_iterable_to_array$1(e)||_non_iterable_rest$1()}function _unsupported_iterable_to_array$1(e,t){if(e){if("string"==typeof e)return _array_like_to_array$1(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if("Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n)return Array.from(n);if("Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return _array_like_to_array$1(e,t)}}var LOG_CATEGORY="[ Federation Runtime ]",parseEntry=function(e,t){var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:SEPARATOR,r=e.split(n),o="development"===getProcessEnv().NODE_ENV&&t,i="*",a=function(e){return e.startsWith("http")||e.includes(MANIFEST_EXT)};if(r.length>=2){var s=_to_array(r),l=s[0],u=s.slice(1),c=o||u.join(n);return a(c)?{name:l,entry:c}:{name:l,version:c||i}}if(1===r.length){var f=_sliced_to_array$1(r,1)[0];return o&&a(o)?{name:f,entry:o}:{name:f,version:o||i}}throw"Invalid entry value: ".concat(e)},logger=new Logger,composeKeyWithSeparator=function(){for(var e=arguments.length,t=Array(e),n=0;n<e;n++)t[n]=arguments[n];return t.length?t.reduce(function(e,t){return t?e?"".concat(e).concat(SEPARATOR).concat(t):t:e},""):""},encodeName=function(e){var t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"",n=arguments.length>2&&void 0!==arguments[2]&&arguments[2];try{var r=n?".js":"";return"".concat(t).concat(e.replace(RegExp("".concat(NameTransformSymbol.AT),"g"),NameTransformMap[NameTransformSymbol.AT]).replace(RegExp("".concat(NameTransformSymbol.HYPHEN),"g"),NameTransformMap[NameTransformSymbol.HYPHEN]).replace(RegExp("".concat(NameTransformSymbol.SLASH),"g"),NameTransformMap[NameTransformSymbol.SLASH])).concat(r)}catch(e){throw e}},decodeName=function(e,t,n){try{var r=e;if(t){if(!r.startsWith(t))return r;r=r.replace(RegExp(t,"g"),"")}return r=r.replace(RegExp("".concat(NameTransformMap[NameTransformSymbol.AT]),"g"),EncodedNameTransformMap[NameTransformMap[NameTransformSymbol.AT]]).replace(RegExp("".concat(NameTransformMap[NameTransformSymbol.SLASH]),"g"),EncodedNameTransformMap[NameTransformMap[NameTransformSymbol.SLASH]]).replace(RegExp("".concat(NameTransformMap[NameTransformSymbol.HYPHEN]),"g"),EncodedNameTransformMap[NameTransformMap[NameTransformSymbol.HYPHEN]]),n&&(r=r.replace(".js","")),r}catch(e){throw e}},generateExposeFilename=function(e,t){if(!e)return"";var n=e;return"."===n&&(n="default_export"),n.startsWith("./")&&(n=n.replace("./","")),encodeName(n,"__federation_expose_",t)},generateShareFilename=function(e,t){return e?encodeName(e,"__federation_shared_",t):""},getResourceUrl=function(e,t){if("getPublicPath"in e){var n=Function(e.getPublicPath)();return"".concat(n).concat(t)}if("publicPath"in e)return"".concat(e.publicPath).concat(t);return console.warn("Can not get resource url, if in debug mode, please ignore",e,t),""},assert=function(e,t){!e&&error(t)},error=function(e){throw Error("".concat(LOG_CATEGORY,": ").concat(e))},warn=function(e){console.warn("".concat(LOG_CATEGORY,": ").concat(e))};function _define_property$1(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function _object_spread$1(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{},r=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter(function(e){return Object.getOwnPropertyDescriptor(n,e).enumerable}))),r.forEach(function(t){_define_property$1(e,t,n[t])})}return e}function ownKeys(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable})),n.push.apply(n,r)}return n}function _object_spread_props(e,t){return t=null!=t?t:{},Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(t)):ownKeys(Object(t)).forEach(function(n){Object.defineProperty(e,n,Object.getOwnPropertyDescriptor(t,n))}),e}var simpleJoinRemoteEntry=function(e,t){if(!e)return t;var n=function(e){if("."===e)return"";if(e.startsWith("./"))return e.replace("./","");if(e.startsWith("/")){var t=e.slice(1);return t.endsWith("/")?t.slice(0,-1):t}return e}(e);return n?n.endsWith("/")?"".concat(n).concat(t):"".concat(n,"/").concat(t):t};function generateSnapshotFromManifest(e){var t,n,r,o,i=arguments.length>1&&void 0!==arguments[1]?arguments[1]:{},a=i.remotes,s=void 0===a?{}:a,l=i.overrides,u=void 0===l?{}:l,c=i.version,f=function(){return"publicPath"in e.metaData?e.metaData.publicPath:e.metaData.getPublicPath},h=Object.keys(u),p={};!Object.keys(s).length&&(p=(null===(o=e.remotes)||void 0===o?void 0:o.reduce(function(e,t){var n,r=t.federationContainerName;return n=h.includes(r)?u[r]:"version"in t?t.version:t.entry,e[r]={matchedVersion:n},e},{}))||{}),Object.keys(s).forEach(function(e){return p[e]={matchedVersion:h.includes(e)?u[e]:s[e]}});var d=e.metaData,m=d.remoteEntry,y=m.path,b=m.name,g=m.type,v=d.types,_=d.buildInfo.buildVersion,E=d.globalName,S=e.exposes,O={version:c||"",buildVersion:_,globalName:E,remoteEntry:simpleJoinRemoteEntry(y,b),remoteEntryType:g,remoteTypes:simpleJoinRemoteEntry(v.path,v.name),remoteTypesZip:v.zip||"",remoteTypesAPI:v.api||"",remotesInfo:p,shared:null==e?void 0:e.shared.map(function(e){return{assets:e.assets,sharedName:e.name,version:e.version}}),modules:null==S?void 0:S.map(function(e){return{moduleName:e.name,modulePath:e.path,assets:e.assets}})};if(null===(t=e.metaData)||void 0===t?void 0:t.prefetchInterface){var I=e.metaData.prefetchInterface;O=_object_spread_props(_object_spread$1({},O),{prefetchInterface:I})}if(null===(n=e.metaData)||void 0===n?void 0:n.prefetchEntry){var w=e.metaData.prefetchEntry,$=w.path,T=w.name,R=w.type;O=_object_spread_props(_object_spread$1({},O),{prefetchEntry:simpleJoinRemoteEntry($,T),prefetchEntryType:R})}return r="publicPath"in e.metaData?_object_spread_props(_object_spread$1({},O),{publicPath:f()}):_object_spread_props(_object_spread$1({},O),{getPublicPath:f()})}function isManifestProvider(e){return!!("remoteEntry"in e&&e.remoteEntry.includes(MANIFEST_EXT))||!1}function asyncGeneratorStep$1(e,t,n,r,o,i,a){try{var s=e[i](a),l=s.value}catch(e){n(e);return}s.done?t(l):Promise.resolve(l).then(r,o)}function _async_to_generator$1(e){return function(){var t=this,n=arguments;return new Promise(function(r,o){var i=e.apply(t,n);function a(e){asyncGeneratorStep$1(i,r,o,a,s,"next",e)}function s(e){asyncGeneratorStep$1(i,r,o,a,s,"throw",e)}a(void 0)})}}function _instanceof(e,t){return null!=t&&"undefined"!=typeof Symbol&&t[Symbol.hasInstance]?!!t[Symbol.hasInstance](e):e instanceof t}function _ts_generator$1(e,t){var n,r,o,i,a={label:0,sent:function(){if(1&o[0])throw o[1];return o[1]},trys:[],ops:[]};return i={next:s(0),throw:s(1),return:s(2)},"function"==typeof Symbol&&(i[Symbol.iterator]=function(){return this}),i;function s(e){return function(t){return l([e,t])}}function l(i){if(n)throw TypeError("Generator is already executing.");for(;a;)try{if(n=1,r&&(o=2&i[0]?r.return:i[0]?r.throw||((o=r.return)&&o.call(r),0):r.next)&&!(o=o.call(r,i[1])).done)return o;switch(r=0,o&&(i=[2&i[0],o.value]),i[0]){case 0:case 1:o=i;break;case 4:return a.label++,{value:i[1],done:!1};case 5:a.label++,r=i[1],i=[0];continue;case 7:i=a.ops.pop(),a.trys.pop();continue;default:if(!(o=(o=a.trys).length>0&&o[o.length-1])&&(6===i[0]||2===i[0])){a=0;continue}if(3===i[0]&&(!o||i[1]>o[0]&&i[1]<o[3])){a.label=i[1];break}if(6===i[0]&&a.label<o[1]){a.label=o[1],o=i;break}if(o&&a.label<o[2]){a.label=o[2],a.ops.push(i);break}o[2]&&a.ops.pop(),a.trys.pop();continue}i=t.call(e,a)}catch(e){i=[6,e],r=0}finally{n=o=0}if(5&i[0])throw i[1];return{value:i[0]?i[1]:void 0,done:!0}}}function safeWrapper(e,t){return _safeWrapper.apply(this,arguments)}function _safeWrapper(){return(_safeWrapper=_async_to_generator$1(function(e,t){var n;return _ts_generator$1(this,function(r){switch(r.label){case 0:return r.trys.push([0,2,,3]),[4,e()];case 1:return[2,r.sent()];case 2:return n=r.sent(),t||warn(n),[2];case 3:return[2]}})})).apply(this,arguments)}function isStaticResourcesEqual(e,t){var n=/^(https?:)?\/\//i;return e.replace(n,"").replace(/\/$/,"")===t.replace(n,"").replace(/\/$/,"")}function createScript(e,t,n,r){for(var o=null,i=!0,a=document.getElementsByTagName("script"),s=0;s<a.length;s++){var l=a[s],u=l.getAttribute("src");if(u&&isStaticResourcesEqual(u,e)){o=l,i=!1;break}}if(!o&&((o=document.createElement("script")).type="text/javascript",o.src=e,r)){var c=r(e);_instanceof(c,HTMLScriptElement)&&(o=c)}n&&Object.keys(n).forEach(function(e){o&&("async"===e||"defer"===e?o[e]=n[e]:o.setAttribute(e,n[e]))});var f=function(e,n){if(o&&(o.onerror=null,o.onload=null,safeWrapper(function(){(null==o?void 0:o.parentNode)&&o.parentNode.removeChild(o)}),e)){var r=e(n);return t(),r}t()};return o.onerror=f.bind(null,o.onerror),o.onload=f.bind(null,o.onload),{script:o,needAttach:i}}function createLink(e,t){for(var n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:{},r=arguments.length>3?arguments[3]:void 0,o=null,i=!0,a=document.getElementsByTagName("link"),s=0;s<a.length;s++){var l=a[s],u=l.getAttribute("href"),c=l.getAttribute("ref");if(u&&isStaticResourcesEqual(u,e)&&c===n.ref){o=l,i=!1;break}}if(!o&&((o=document.createElement("link")).setAttribute("href",e),r)){var f=r(e);_instanceof(f,HTMLLinkElement)&&(o=f)}n&&Object.keys(n).forEach(function(e){o&&o.setAttribute(e,n[e])});var h=function(e,n){if(o&&(o.onerror=null,o.onload=null,safeWrapper(function(){(null==o?void 0:o.parentNode)&&o.parentNode.removeChild(o)}),e)){var r=e(n);return t(),r}t()};return o.onerror=h.bind(null,o.onerror),o.onload=h.bind(null,o.onload),{link:o,needAttach:i}}function loadScript(e,t){var n=t.attrs,r=t.createScriptHook;return new Promise(function(t,o){var i=createScript(e,t,n,r),a=i.script;i.needAttach&&document.getElementsByTagName("head")[0].appendChild(a)})}function _array_like_to_array(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=Array(t);n<t;n++)r[n]=e[n];return r}function _array_with_holes(e){if(Array.isArray(e))return e}function asyncGeneratorStep(e,t,n,r,o,i,a){try{var s=e[i](a),l=s.value}catch(e){n(e);return}s.done?t(l):Promise.resolve(l).then(r,o)}function _async_to_generator(e){return function(){var t=this,n=arguments;return new Promise(function(r,o){var i=e.apply(t,n);function a(e){asyncGeneratorStep(i,r,o,a,s,"next",e)}function s(e){asyncGeneratorStep(i,r,o,a,s,"throw",e)}a(void 0)})}}function _iterable_to_array_limit(e,t){var n,r,o=null==e?null:"undefined"!=typeof Symbol&&e[Symbol.iterator]||e["@@iterator"];if(null!=o){var i=[],a=!0,s=!1;try{for(o=o.call(e);!(a=(n=o.next()).done)&&(i.push(n.value),!t||i.length!==t);a=!0);}catch(e){s=!0,r=e}finally{try{!a&&null!=o.return&&o.return()}finally{if(s)throw r}}return i}}function _non_iterable_rest(){throw TypeError("Invalid attempt to destructure non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function _sliced_to_array(e,t){return _array_with_holes(e)||_iterable_to_array_limit(e,t)||_unsupported_iterable_to_array(e,t)||_non_iterable_rest()}function _unsupported_iterable_to_array(e,t){if(e){if("string"==typeof e)return _array_like_to_array(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if("Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n)return Array.from(n);if("Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return _array_like_to_array(e,t)}}function _ts_generator(e,t){var n,r,o,i,a={label:0,sent:function(){if(1&o[0])throw o[1];return o[1]},trys:[],ops:[]};return i={next:s(0),throw:s(1),return:s(2)},"function"==typeof Symbol&&(i[Symbol.iterator]=function(){return this}),i;function s(e){return function(t){return l([e,t])}}function l(i){if(n)throw TypeError("Generator is already executing.");for(;a;)try{if(n=1,r&&(o=2&i[0]?r.return:i[0]?r.throw||((o=r.return)&&o.call(r),0):r.next)&&!(o=o.call(r,i[1])).done)return o;switch(r=0,o&&(i=[2&i[0],o.value]),i[0]){case 0:case 1:o=i;break;case 4:return a.label++,{value:i[1],done:!1};case 5:a.label++,r=i[1],i=[0];continue;case 7:i=a.ops.pop(),a.trys.pop();continue;default:if(!(o=(o=a.trys).length>0&&o[o.length-1])&&(6===i[0]||2===i[0])){a=0;continue}if(3===i[0]&&(!o||i[1]>o[0]&&i[1]<o[3])){a.label=i[1];break}if(6===i[0]&&a.label<o[1]){a.label=o[1],o=i;break}if(o&&a.label<o[2]){a.label=o[2],a.ops.push(i);break}o[2]&&a.ops.pop(),a.trys.pop();continue}i=t.call(e,a)}catch(e){i=[6,e],r=0}finally{n=o=0}if(5&i[0])throw i[1];return{value:i[0]?i[1]:void 0,done:!0}}}function importNodeModule(e){if(!e)throw Error("import specifier is required");return Function("name","return import(name)")(e).then(function(e){return e.default}).catch(function(t){throw console.error("Error importing module ".concat(e,":"),t),t})}function createScriptNode(url,cb,attrs,createScriptHook){if(createScriptHook){var urlObj,hookResult=createScriptHook(url);hookResult&&"object"==typeof hookResult&&"url"in hookResult&&(url=hookResult.url)}try{urlObj=new URL(url)}catch(e){console.error("Error constructing URL:",e),cb(Error("Invalid URL: ".concat(e)));return}var getFetch=function(){var e=_async_to_generator(function(){var e;return _ts_generator(this,function(t){switch(t.label){case 0:if("undefined"!=typeof fetch)return[3,2];return[4,importNodeModule("node-fetch")];case 1:return[2,(null==(e=t.sent())?void 0:e.default)||e];case 2:return[2,fetch];case 3:return[2]}})});return function(){return e.apply(this,arguments)}}();console.log("fetching",urlObj.href),getFetch().then(function(f){f(urlObj.href).then(function(e){return e.text()}).then(function(){var _ref=_async_to_generator(function(data){var _ref,path,vm,scriptContext,urlDirname,filename,script,exportedInterface,container;return _ts_generator(this,function(_state){switch(_state.label){case 0:return[4,Promise.all([importNodeModule("path"),importNodeModule("vm")])];case 1:_ref=_sliced_to_array.apply(void 0,[_state.sent(),2]),path=_ref[0],vm=_ref[1],scriptContext={exports:{},module:{exports:{}}},urlDirname=urlObj.pathname.split("/").slice(0,-1).join("/"),filename=path.basename(urlObj.pathname);try{if(script=new vm.Script("(function(exports, module, require, __dirname, __filename) {".concat(data,"\n})"),filename),script.runInThisContext()(scriptContext.exports,scriptContext.module,eval("require"),urlDirname,filename),exportedInterface=scriptContext.module.exports||scriptContext.exports,attrs&&exportedInterface&&attrs.globalName)return container=exportedInterface[attrs.globalName]||exportedInterface,cb(void 0,container),[2];cb(void 0,exportedInterface)}catch(e){cb(Error("Script execution error: ".concat(e)))}return[2]}})});return function(e){return _ref.apply(this,arguments)}}()).catch(function(e){cb(e)})})}function loadScriptNode(e,t){return new Promise(function(n,r){createScriptNode(e,function(e,o){if(e)r(e);else{var i,a,s=(null==t?void 0:null===(i=t.attrs)||void 0===i?void 0:i.globalName)||"__FEDERATION_".concat(null==t?void 0:null===(a=t.attrs)||void 0===a?void 0:a.name,":custom__");n(globalThis[s]=o)}},t.attrs,t.createScriptHook)})}function _define_property(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function _object_spread(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{},r=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter(function(e){return Object.getOwnPropertyDescriptor(n,e).enumerable}))),r.forEach(function(t){_define_property(e,t,n[t])})}return e}function _type_of(e){return e&&"undefined"!=typeof Symbol&&e.constructor===Symbol?"symbol":typeof e}function normalizeOptions(e,t,n){return function(r){if(!1===r)return!1;if(void 0===r)return!!e&&t;if(!0===r)return t;if(r&&"object"==typeof r)return _object_spread({},t,r);throw Error("Unexpected type for `".concat(n,"`, expect boolean/undefined/object, got: ").concat(void 0===r?"undefined":_type_of(r)))}}exports.BROWSER_LOG_KEY=BROWSER_LOG_KEY,exports.BROWSER_LOG_VALUE=BROWSER_LOG_VALUE,exports.EncodedNameTransformMap=EncodedNameTransformMap,exports.FederationModuleManifest=FederationModuleManifest,exports.Logger=Logger,exports.MANIFEST_EXT=MANIFEST_EXT,exports.MFModuleType=MFModuleType,exports.MODULE_DEVTOOL_IDENTIFIER=MODULE_DEVTOOL_IDENTIFIER,exports.ManifestFileName=ManifestFileName,exports.NameTransformMap=NameTransformMap,exports.NameTransformSymbol=NameTransformSymbol,exports.SEPARATOR=SEPARATOR,exports.StatsFileName=StatsFileName,exports.assert=assert,exports.composeKeyWithSeparator=composeKeyWithSeparator,exports.containerPlugin=ContainerPlugin,exports.containerReferencePlugin=ContainerReferencePlugin,exports.createLink=createLink,exports.createScript=createScript,exports.createScriptNode=createScriptNode,exports.decodeName=decodeName,exports.encodeName=encodeName,exports.error=error,exports.generateExposeFilename=generateExposeFilename,exports.generateShareFilename=generateShareFilename,exports.generateSnapshotFromManifest=generateSnapshotFromManifest,exports.getProcessEnv=getProcessEnv,exports.getResourceUrl=getResourceUrl,exports.isBrowserEnv=isBrowserEnv,exports.isDebugMode=isDebugMode,exports.isManifestProvider=isManifestProvider,exports.isStaticResourcesEqual=isStaticResourcesEqual,exports.loadScript=loadScript,exports.loadScriptNode=loadScriptNode,exports.logger=logger,exports.moduleFederationPlugin=ModuleFederationPlugin,exports.normalizeOptions=normalizeOptions,exports.parseEntry=parseEntry,exports.safeWrapper=safeWrapper,exports.sharePlugin=SharePlugin,exports.simpleJoinRemoteEntry=simpleJoinRemoteEntry,exports.warn=warn},29705:function(e,t){"use strict";Object.defineProperty(t,"__esModule",{value:!0});var n="ENCODE_NAME_PREFIX",r=["script"];t.ENCODE_NAME_PREFIX=n,t.FEDERATION_SUPPORTED_TYPES=r},37699:function(e,t,n){"use strict";var r=n(90035),o=n(5580),i=n(29705);function a(e){if(!!e.S&&!e.federation.hasAttachShareScopeMap&&!!e.federation.instance&&!!e.federation.instance.shareScopeMap)e.S=e.federation.instance.shareScopeMap,e.federation.hasAttachShareScopeMap=!0}function s(e){var t=e.chunkId,n=e.promises,r=e.chunkMapping,s=e.idToExternalAndNameMapping,l=e.webpackRequire,u=e.idToRemoteMap;a(l),l.o(r,t)&&r[t].forEach(function(e){var t=l.R;!t&&(t=[]);var r=s[e],a=u[e];if(!(t.indexOf(r)>=0)){if(t.push(r),r.p)return n.push(r.p);var c=function(t){!t&&(t=Error("Container missing")),"string"==typeof t.message&&(t.message+='\nwhile loading "'.concat(r[1],'" from ').concat(r[2])),l.m[e]=function(){throw t},r.p=0},f=function(e,t,o,i,a,s){try{var l=e(t,o);if(!l||!l.then)return a(l,i,s);var u=l.then(function(e){return a(e,i)},c);if(!s)return u;n.push(r.p=u)}catch(e){c(e)}},h=function(e,t,n){return e?f(l.I,r[0],0,e,p,n):c()},p=function(e,n,o){return f(n.get,r[1],t,0,d,o)},d=function(t){r.p=1,l.m[e]=function(e){e.exports=t()}},m=function(){try{var e=o.decodeName(a[0].name,i.ENCODE_NAME_PREFIX)+r[1].slice(1);return l.federation.instance.loadRemote(e,{loadFactory:!1,from:"build"})}catch(e){c(e)}};1===a.length&&["script"].includes(a[0].externalType)&&a[0].name?f(m,r[2],0,0,d,1):f(l,r[2],0,0,h,1)}})}function l(e){var t=e.chunkId,n=e.promises,r=e.chunkMapping,o=e.installedModules,i=e.moduleToHandlerMapping,s=e.webpackRequire;a(s),s.o(r,t)&&r[t].forEach(function(e){if(s.o(o,e))return n.push(o[e]);var t=function(t){o[e]=0,s.m[e]=function(n){delete s.c[e],n.exports=t()}},r=function(t){delete o[e],s.m[e]=function(n){throw delete s.c[e],t}};try{var a=s.federation.instance;if(!a)throw Error("Federation instance not found!");var l=i[e],u=l.shareKey,c=l.getter,f=l.shareInfo,h=a.loadShare(u,{customShareInfo:f}).then(function(e){return!1===e?c():e});h.then?n.push(o[e]=h.then(t).catch(r)):t(h)}catch(e){r(e)}})}function u(e){var t=e.shareScopeName,n=e.webpackRequire,r=e.initPromises,o=e.initTokens,s=e.initScope;!s&&(s=[]);var l=o[t];if(!l&&(l=o[t]={}),!(s.indexOf(l)>=0)){s.push(l);var u=r[t];if(u)return u;var c=function(e){return"undefined"!=typeof console&&console.warn&&console.warn(e)},f=function(e){var r=function(e){return c("Initialization of sharing external failed: "+e)};try{var o=n(e);if(!o)return;var i=function(e){return e&&e.init&&e.init(n.S[t],s)};if(o.then)return h.push(o.then(i,r));var a=i(o);if(a&&"boolean"!=typeof a&&a.then)return h.push(a.catch(r))}catch(e){r(e)}},h=n.federation.instance.initializeSharing(t);a(n);var p=n.federation.bundlerRuntimeOptions.remotes;return(p&&Object.keys(p.idToRemoteMap).forEach(function(e){var t=p.idToRemoteMap[e],n=p.idToExternalAndNameMapping[e][2];if(t.length>1)f(n);else if(1===t.length){var r=t[0];!i.FEDERATION_SUPPORTED_TYPES.includes(r.externalType)&&f(n)}}),h.length)?r[t]=Promise.all(h).then(function(){return r[t]=!0}):r[t]=!0}}function c(e){var t=e.moduleId,n=e.moduleToHandlerMapping,r=e.webpackRequire.federation.instance;if(!r)throw Error("Federation instance not found!");var o=n[t],i=o.shareKey,a=o.shareInfo;return r.loadShareSync(i,{customShareInfo:a})}function f(e){var t=e.moduleToHandlerMapping,n=e.webpackRequire,r=e.installedModules;e.initialConsumes.forEach(function(e){n.m[e]=function(o){r[e]=0,delete n.c[e];var i=c({moduleId:e,moduleToHandlerMapping:t,webpackRequire:n});if("function"!=typeof i)throw Error("Shared module is not available for eager consumption: ".concat(e));o.exports=i()}})}function h(e,t,n){return t in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function p(e){for(var t=1;t<arguments.length;t++){var n=null!=arguments[t]?arguments[t]:{},r=Object.keys(n);"function"==typeof Object.getOwnPropertySymbols&&(r=r.concat(Object.getOwnPropertySymbols(n).filter(function(e){return Object.getOwnPropertyDescriptor(n,e).enumerable}))),r.forEach(function(t){h(e,t,n[t])})}return e}function d(e){var t=e.webpackRequire,n=e.shareScope,r=e.initScope,o=e.shareScopeKey,i=e.remoteEntryInitOptions;if(!!t.S){if(t.federation&&t.federation.instance&&t.federation.initOptions){var a=t.federation.instance,s=o||"default";return a.initOptions(p({name:t.federation.initOptions.name,remotes:[]},i)),a.initShareScopeMap(s,n),t.federation.attachShareScopeMap&&t.federation.attachShareScopeMap(t),t.I(s,r)}}}var m={runtime:function e(e){if(e&&e.__esModule)return e;var t=Object.create(null);return e&&Object.keys(e).forEach(function(n){if("default"!==n){var r=Object.getOwnPropertyDescriptor(e,n);Object.defineProperty(t,n,r.get?r:{enumerable:!0,get:function(){return e[n]}})}}),t.default=e,Object.freeze(t)}(r),instance:void 0,initOptions:void 0,bundlerRuntime:{remotes:s,consumes:l,I:u,S:{},installInitialConsumes:f,initContainerEntry:d},attachShareScopeMap:a,bundlerRuntimeOptions:{}};e.exports=m},14224:function(e){var t,n,r,o=e.exports={};function i(){throw Error("setTimeout has not been defined")}function a(){throw Error("clearTimeout has not been defined")}function s(e){if(t===setTimeout)return setTimeout(e,0);if((t===i||!t)&&setTimeout)return t=setTimeout,setTimeout(e,0);try{return t(e,0)}catch(n){try{return t.call(null,e,0)}catch(n){return t.call(this,e,0)}}}function l(e){if(n===clearTimeout)return clearTimeout(e);if((n===a||!n)&&clearTimeout)return n=clearTimeout,clearTimeout(e);try{return n(e)}catch(t){try{return n.call(null,e)}catch(t){return n.call(this,e)}}}!function(){try{t="function"==typeof setTimeout?setTimeout:i}catch(e){t=i}try{n="function"==typeof clearTimeout?clearTimeout:a}catch(e){n=a}}();var u=[],c=!1,f=-1;function h(){if(!!c&&!!r)c=!1,r.length?u=r.concat(u):f=-1,u.length&&p()}function p(){if(!c){var e=s(h);c=!0;for(var t=u.length;t;){for(r=u,u=[];++f<t;)r&&r[f].run();f=-1,t=u.length}r=null,c=!1,l(e)}}function d(e,t){this.fun=e,this.array=t}function m(){}o.nextTick=function(e){var t=Array(arguments.length-1);if(arguments.length>1)for(var n=1;n<arguments.length;n++)t[n-1]=arguments[n];u.push(new d(e,t)),1===u.length&&!c&&s(p)},d.prototype.run=function(){this.fun.apply(null,this.array)},o.title="browser",o.browser=!0,o.env={},o.argv=[],o.version="",o.versions={},o.on=m,o.addListener=m,o.once=m,o.off=m,o.removeListener=m,o.removeAllListeners=m,o.emit=m,o.prependListener=m,o.prependOnceListener=m,o.listeners=function(e){return[]},o.binding=function(e){throw Error("process.binding is not supported")},o.cwd=function(){return"/"},o.chdir=function(e){throw Error("process.chdir is not supported")},o.umask=function(){return 0}},67693:function(e,t,n){"use strict";var r=n(37699),o=n.n(r);function i(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=Array(t);n<t;n++)r[n]=e[n];return r}function a(e){if(Array.isArray(e))return e}function s(e){if(Array.isArray(e))return i(e)}function l(e){if("undefined"!=typeof Symbol&&null!=e[Symbol.iterator]||null!=e["@@iterator"])return Array.from(e)}function u(e,t){var n,r,o=null==e?null:"undefined"!=typeof Symbol&&e[Symbol.iterator]||e["@@iterator"];if(null!=o){var i=[],a=!0,s=!1;try{for(o=o.call(e);!(a=(n=o.next()).done)&&(i.push(n.value),!t||i.length!==t);a=!0);}catch(e){s=!0,r=e}finally{try{!a&&null!=o.return&&o.return()}finally{if(s)throw r}}return i}}function c(){throw TypeError("Invalid attempt to destructure non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function f(){throw TypeError("Invalid attempt to spread non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function h(e,t){return a(e)||u(e,t)||m(e,t)||c()}function p(e){return s(e)||l(e)||m(e)||f()}function d(e){return e&&"undefined"!=typeof Symbol&&e.constructor===Symbol?"symbol":typeof e}function m(e,t){if(e){if("string"==typeof e)return i(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if("Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n)return Array.from(n);if("Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return i(e,t)}}var y=[],b={},g="centreon";if((n.initializeSharingData||n.initializeExposesData)&&n.federation){var v,_,E,S,O,I,w,$,T,R,A,N,P=function(e,t,n){e&&e[t]&&(e[t]=n)},M=function(e,t,n){var r,o,i,a,s,l,u,c=n();Array.isArray(c)?(null!==(a=(o=e)[i=t])&&void 0!==a||(o[i]=[]),(r=e[t]).push.apply(r,p(c))):(void 0===c?"undefined":d(c))==="object"&&null!==c&&(null!==(u=(s=e)[l=t])&&void 0!==u||(s[l]={}),Object.assign(e[t],c))},j=function(e,t,n){var r,o,i;null!==(i=(r=e)[o=t])&&void 0!==i||(r[o]=n())},k=null!==($=null===(v=n.remotesLoadingData)||void 0===v?void 0:v.chunkMapping)&&void 0!==$?$:{},D=null!==(T=null===(_=n.remotesLoadingData)||void 0===_?void 0:_.moduleIdToRemoteDataMapping)&&void 0!==T?T:{},L=null!==(R=null===(E=n.initializeSharingData)||void 0===E?void 0:E.scopeToSharingDataMapping)&&void 0!==R?R:{},x=null!==(A=null===(S=n.consumesLoadingData)||void 0===S?void 0:S.chunkMapping)&&void 0!==A?A:{},F=null!==(N=null===(O=n.consumesLoadingData)||void 0===O?void 0:O.moduleIdToConsumeDataMapping)&&void 0!==N?N:{},C={},H=[],G=[],U=null===(I=n.initializeExposesData)||void 0===I?void 0:I.shareScope;for(var B in o())n.federation[B]=o()[B];j(n.federation,"consumesLoadingModuleToHandlerMapping",function(){var e={},t=!0,n=!1,r=void 0;try{for(var o,i=Object.entries(F)[Symbol.iterator]();!(t=(o=i.next()).done);t=!0){var a=h(o.value,2),s=a[0],l=a[1];e[s]={getter:l.fallback,shareInfo:{shareConfig:{fixedDependencies:!1,requiredVersion:l.requiredVersion,strictVersion:l.strictVersion,singleton:l.singleton,eager:l.eager},scope:[l.shareScope]},shareKey:l.shareKey}}}catch(e){n=!0,r=e}finally{try{!t&&null!=i.return&&i.return()}finally{if(n)throw r}}return e}),j(n.federation,"initOptions",function(){return{}}),j(n.federation.initOptions,"name",function(){return g}),j(n.federation.initOptions,"shared",function(){var e={},t=!0,n=!1,r=void 0;try{for(var o,i=Object.entries(L)[Symbol.iterator]();!(t=(o=i.next()).done);t=!0){var a=h(o.value,2),s=a[0],l=a[1],u=!0,c=!1,f=void 0;try{for(var p,m=l[Symbol.iterator]();!(u=(p=m.next()).done);u=!0){var y=p.value;if((void 0===y?"undefined":d(y))==="object"&&null!==y){var b=y.name,g=y.version,v=y.factory;y.eager;var _={version:g,scope:[s],get:v};e[b]?e[b].push(_):e[b]=[_]}}}catch(e){c=!0,f=e}finally{try{!u&&null!=m.return&&m.return()}finally{if(c)throw f}}}}catch(e){n=!0,r=e}finally{try{!t&&null!=i.return&&i.return()}finally{if(n)throw r}}return e}),M(n.federation.initOptions,"remotes",function(){return Object.values(b).flat().filter(function(e){return"script"===e.externalType})}),M(n.federation.initOptions,"plugins",function(){return y}),j(n.federation,"bundlerRuntimeOptions",function(){return{}}),j(n.federation.bundlerRuntimeOptions,"remotes",function(){return{}}),j(n.federation.bundlerRuntimeOptions.remotes,"chunkMapping",function(){return k}),j(n.federation.bundlerRuntimeOptions.remotes,"idToExternalAndNameMapping",function(){var e={},t=!0,n=!1,r=void 0;try{for(var o,i=Object.entries(D)[Symbol.iterator]();!(t=(o=i.next()).done);t=!0){var a=h(o.value,2),s=a[0],l=a[1];e[s]=[l.shareScope,l.name,l.externalModuleId,l.remoteName]}}catch(e){n=!0,r=e}finally{try{!t&&null!=i.return&&i.return()}finally{if(n)throw r}}return e}),j(n.federation.bundlerRuntimeOptions.remotes,"webpackRequire",function(){return n}),M(n.federation.bundlerRuntimeOptions.remotes,"idToRemoteMap",function(){var e={},t=!0,n=!1,r=void 0;try{for(var o,i=Object.entries(D)[Symbol.iterator]();!(t=(o=i.next()).done);t=!0){var a=h(o.value,2),s=a[0],l=b[a[1].remoteName];l&&(e[s]=l)}}catch(e){n=!0,r=e}finally{try{!t&&null!=i.return&&i.return()}finally{if(n)throw r}}return e}),P(n,"S",n.federation.bundlerRuntime.S),n.federation.attachShareScopeMap&&n.federation.attachShareScopeMap(n),P(n.f,"remotes",function(e,t){return n.federation.bundlerRuntime.remotes({chunkId:e,promises:t,chunkMapping:k,idToExternalAndNameMapping:n.federation.bundlerRuntimeOptions.remotes.idToExternalAndNameMapping,idToRemoteMap:n.federation.bundlerRuntimeOptions.remotes.idToRemoteMap,webpackRequire:n})}),P(n.f,"consumes",function(e,t){return n.federation.bundlerRuntime.consumes({chunkId:e,promises:t,chunkMapping:x,moduleToHandlerMapping:n.federation.consumesLoadingModuleToHandlerMapping,installedModules:C,webpackRequire:n})}),P(n,"I",function(e,t){return n.federation.bundlerRuntime.I({shareScopeName:e,initScope:t,initPromises:H,initTokens:G,webpackRequire:n})}),P(n,"initContainer",function(e,t,r){return n.federation.bundlerRuntime.initContainerEntry({shareScope:e,initScope:t,remoteEntryInitOptions:r,shareScopeKey:U,webpackRequire:n})}),P(n,"getContainer",function(e,t){var r=n.initializeExposesData.moduleMap;return n.R=t,t=Object.prototype.hasOwnProperty.call(r,e)?r[e]():Promise.resolve().then(function(){throw Error('Module "'+e+'" does not exist in container.')}),n.R=void 0,t}),n.federation.instance=n.federation.runtime.init(n.federation.initOptions),(null===(w=n.consumesLoadingData)||void 0===w?void 0:w.initialConsumes)&&n.federation.bundlerRuntime.installInitialConsumes({webpackRequire:n,installedModules:C,initialConsumes:n.consumesLoadingData.initialConsumes,moduleToHandlerMapping:n.federation.consumesLoadingModuleToHandlerMapping})}}}]);
//# sourceMappingURL=6556.9ee6bfa19346bf01410d.e8e7040a846d504f76a8.js.map