"use strict";(self.webpackChunkcentreon=self.webpackChunkcentreon||[]).push([["4269"],{83409:function(e,r,t){var n=t(88619);r.default=void 0;var o=n(t(14420)),a=t(52676);r.default=(0,o.default)((0,a.jsx)("path",{d:"M6.23 20.23 8 22l10-10L8 2 6.23 3.77 14.46 12z"}),"ArrowForwardIos")},35459:function(e,r,t){var n=t(88619);r.default=void 0;var o=n(t(14420)),a=t(52676);r.default=(0,o.default)((0,a.jsx)("path",{d:"M12 5.83 15.17 9l1.41-1.41L12 3 7.41 7.59 8.83 9zm0 12.34L8.83 15l-1.41 1.41L12 21l4.59-4.59L15.17 15z"}),"UnfoldMore")},76247:function(e,r,t){var n=t(59869),o=t(83192),a=t(1975),i=t(54461),l=t(46436),c=t(44445),u=t(86365),s=t(99045),f=t(78653),d=t(20524),b=t(52676);let m=["children","className"],p=e=>{let{classes:r}=e,t=(0,l.Z)({root:["root"]},d.i,r);return(0,o.Z)({},r,t)},y=(0,c.ZP)(f.Z,{shouldForwardProp:e=>(0,u.Z)(e)||"classes"===e,name:"MuiDialogContentText",slot:"Root",overridesResolver:(e,r)=>r.root})({}),g=a.forwardRef(function(e,r){let t=(0,s.i)({props:e,name:"MuiDialogContentText"}),{className:a}=t,l=(0,n.Z)(t,m),c=p(l);return(0,b.jsx)(y,(0,o.Z)({component:"p",variant:"body1",color:"text.secondary",ref:r,ownerState:l,className:(0,i.Z)(c.root,a)},t,{classes:c}))});r.Z=g},20524:function(e,r,t){t.d(r,{i:function(){return a}});var n=t(94078),o=t(82626);function a(e){return(0,o.ZP)("MuiDialogContentText",e)}let i=(0,n.Z)("MuiDialogContentText",["root"]);r.Z=i},5037:function(e,r,t){var n=t(59869),o=t(83192),a=t(1975),i=t(54461),l=t(46436),c=t(56517),u=t(44012),s=t(3273),f=t(81775),d=t(44445),b=t(99045),m=t(24329),p=t(52676);let y=["className","color","value","valueBuffer","variant"],g=e=>e,v,O,h,P,j,Z,w=(0,c.F4)(v||(v=g`
  0% {
    left: -35%;
    right: 100%;
  }

  60% {
    left: 100%;
    right: -90%;
  }

  100% {
    left: 100%;
    right: -90%;
  }
`)),C=(0,c.F4)(O||(O=g`
  0% {
    left: -200%;
    right: 100%;
  }

  60% {
    left: 107%;
    right: -8%;
  }

  100% {
    left: 107%;
    right: -8%;
  }
`)),S=(0,c.F4)(h||(h=g`
  0% {
    opacity: 1;
    background-position: 0 -23px;
  }

  60% {
    opacity: 0;
    background-position: 0 -23px;
  }

  100% {
    opacity: 1;
    background-position: -200px -23px;
  }
`)),E=e=>{let{classes:r,variant:t,color:n}=e,o={root:["root",`color${(0,f.Z)(n)}`,t],dashed:["dashed",`dashedColor${(0,f.Z)(n)}`],bar1:["bar",`barColor${(0,f.Z)(n)}`,("indeterminate"===t||"query"===t)&&"bar1Indeterminate","determinate"===t&&"bar1Determinate","buffer"===t&&"bar1Buffer"],bar2:["bar","buffer"!==t&&`barColor${(0,f.Z)(n)}`,"buffer"===t&&`color${(0,f.Z)(n)}`,("indeterminate"===t||"query"===t)&&"bar2Indeterminate","buffer"===t&&"bar2Buffer"]};return(0,l.Z)(o,m.E,r)},x=(e,r)=>"inherit"===r?"currentColor":e.vars?e.vars.palette.LinearProgress[`${r}Bg`]:"light"===e.palette.mode?(0,u.lighten)(e.palette[r].main,.62):(0,u.darken)(e.palette[r].main,.5),D=(0,d.ZP)("span",{name:"MuiLinearProgress",slot:"Root",overridesResolver:(e,r)=>{let{ownerState:t}=e;return[r.root,r[`color${(0,f.Z)(t.color)}`],r[t.variant]]}})(({ownerState:e,theme:r})=>(0,o.Z)({position:"relative",overflow:"hidden",display:"block",height:4,zIndex:0,"@media print":{colorAdjust:"exact"},backgroundColor:x(r,e.color)},"inherit"===e.color&&"buffer"!==e.variant&&{backgroundColor:"none","&::before":{content:'""',position:"absolute",left:0,top:0,right:0,bottom:0,backgroundColor:"currentColor",opacity:.3}},"buffer"===e.variant&&{backgroundColor:"transparent"},"query"===e.variant&&{transform:"rotate(180deg)"})),I=(0,d.ZP)("span",{name:"MuiLinearProgress",slot:"Dashed",overridesResolver:(e,r)=>{let{ownerState:t}=e;return[r.dashed,r[`dashedColor${(0,f.Z)(t.color)}`]]}})(({ownerState:e,theme:r})=>{let t=x(r,e.color);return(0,o.Z)({position:"absolute",marginTop:0,height:"100%",width:"100%"},"inherit"===e.color&&{opacity:.3},{backgroundImage:`radial-gradient(${t} 0%, ${t} 16%, transparent 42%)`,backgroundSize:"10px 10px",backgroundPosition:"0 -23px"})},(0,c.iv)(P||(P=g`
    animation: ${0} 3s infinite linear;
  `),S)),k=(0,d.ZP)("span",{name:"MuiLinearProgress",slot:"Bar1",overridesResolver:(e,r)=>{let{ownerState:t}=e;return[r.bar,r[`barColor${(0,f.Z)(t.color)}`],("indeterminate"===t.variant||"query"===t.variant)&&r.bar1Indeterminate,"determinate"===t.variant&&r.bar1Determinate,"buffer"===t.variant&&r.bar1Buffer]}})(({ownerState:e,theme:r})=>(0,o.Z)({width:"100%",position:"absolute",left:0,bottom:0,top:0,transition:"transform 0.2s linear",transformOrigin:"left",backgroundColor:"inherit"===e.color?"currentColor":(r.vars||r).palette[e.color].main},"determinate"===e.variant&&{transition:"transform .4s linear"},"buffer"===e.variant&&{zIndex:1,transition:"transform .4s linear"}),({ownerState:e})=>("indeterminate"===e.variant||"query"===e.variant)&&(0,c.iv)(j||(j=g`
      width: auto;
      animation: ${0} 2.1s cubic-bezier(0.65, 0.815, 0.735, 0.395) infinite;
    `),w)),A=(0,d.ZP)("span",{name:"MuiLinearProgress",slot:"Bar2",overridesResolver:(e,r)=>{let{ownerState:t}=e;return[r.bar,r[`barColor${(0,f.Z)(t.color)}`],("indeterminate"===t.variant||"query"===t.variant)&&r.bar2Indeterminate,"buffer"===t.variant&&r.bar2Buffer]}})(({ownerState:e,theme:r})=>(0,o.Z)({width:"100%",position:"absolute",left:0,bottom:0,top:0,transition:"transform 0.2s linear",transformOrigin:"left"},"buffer"!==e.variant&&{backgroundColor:"inherit"===e.color?"currentColor":(r.vars||r).palette[e.color].main},"inherit"===e.color&&{opacity:.3},"buffer"===e.variant&&{backgroundColor:x(r,e.color),transition:"transform .4s linear"}),({ownerState:e})=>("indeterminate"===e.variant||"query"===e.variant)&&(0,c.iv)(Z||(Z=g`
      width: auto;
      animation: ${0} 2.1s cubic-bezier(0.165, 0.84, 0.44, 1) 1.15s infinite;
    `),C)),N=a.forwardRef(function(e,r){let t=(0,b.i)({props:e,name:"MuiLinearProgress"}),{className:a,color:l="primary",value:c,valueBuffer:u,variant:f="indeterminate"}=t,d=(0,n.Z)(t,y),m=(0,o.Z)({},t,{color:l,variant:f}),g=E(m),v=(0,s.V)(),O={},h={bar1:{},bar2:{}};if("determinate"===f||"buffer"===f){if(void 0!==c){O["aria-valuenow"]=Math.round(c),O["aria-valuemin"]=0,O["aria-valuemax"]=100;let e=c-100;v&&(e=-e),h.bar1.transform=`translateX(${e}%)`}}if("buffer"===f){if(void 0!==u){let e=(u||0)-100;v&&(e=-e),h.bar2.transform=`translateX(${e}%)`}}return(0,p.jsxs)(D,(0,o.Z)({className:(0,i.Z)(g.root,a),ownerState:m,role:"progressbar"},O,{ref:r},d,{children:["buffer"===f?(0,p.jsx)(I,{className:g.dashed,ownerState:m}):null,(0,p.jsx)(k,{className:g.bar1,ownerState:m,style:h.bar1}),"determinate"===f?null:(0,p.jsx)(A,{className:g.bar2,ownerState:m,style:h.bar2})]}))});r.Z=N},24329:function(e,r,t){t.d(r,{E:function(){return a}});var n=t(94078),o=t(82626);function a(e){return(0,o.ZP)("MuiLinearProgress",e)}let i=(0,n.Z)("MuiLinearProgress",["root","colorPrimary","colorSecondary","determinate","indeterminate","buffer","query","dashed","dashedColorPrimary","dashedColorSecondary","bar","barColorPrimary","barColorSecondary","bar1Indeterminate","bar1Determinate","bar1Buffer","bar2Indeterminate","bar2Buffer"]);r.Z=i},32767:function(e,r,t){var n=t(87146),o=t(63128),a=t(1975),i=(0,n.ZL)()(function(){return{alignCenter:{alignItems:"center",display:"grid",height:"100%",justifyContent:"center",width:"100%"}}});r.Z=function(e){var r,t,n,l=e.loading,c=e.children,u=e.loadingIndicatorSize,s=e.alignCenter,f=e.className,d=e.loadingContainerClassname,b=i(),m=b.classes,p=b.cx;if(l){;return a.createElement("div",{className:p((r={},t=m.alignCenter,n=void 0===s||s,t in r?Object.defineProperty(r,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):r[t]=n,r),d)},a.createElement(o.Z,{className:f,size:void 0===u?void 0:u}))}return c}},75541:function(e,r,t){var n=t(76247),o=t(78653),a=t(8540),i=t(1975);r.Z=function(e){var r=e.labelMessage,t=e.labelSecondMessage,l=e.children,c=e.restCancelButtonProps,u=e.restConfirmButtonProps,s=function(e,r){if(null==e)return{};var t,n,o=function(e,r){if(null==e)return{};var t,n,o={},a=Object.keys(e);for(n=0;n<a.length;n++)t=a[n],!(r.indexOf(t)>=0)&&(o[t]=e[t]);return o}(e,r);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(n=0;n<a.length;n++){if(t=a[n],!(r.indexOf(t)>=0))Object.prototype.propertyIsEnumerable.call(e,t)&&(o[t]=e[t])}}return o}(e,["labelMessage","labelSecondMessage","children","restCancelButtonProps","restConfirmButtonProps"]);return i.createElement(a.Z,function(e){for(var r=1;r<arguments.length;r++){var t=null!=arguments[r]?arguments[r]:{},n=Object.keys(t);"function"==typeof Object.getOwnPropertySymbols&&(n=n.concat(Object.getOwnPropertySymbols(t).filter(function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.forEach(function(r){var n,o,a;n=e,o=r,a=t[r],o in n?Object.defineProperty(n,o,{value:a,enumerable:!0,configurable:!0,writable:!0}):n[o]=a})}return e}({restCancelButtonProps:c,restConfirmButtonProps:u},s),i.createElement(n.Z,null,r&&i.createElement(o.Z,null,r),t&&i.createElement(o.Z,null,t),l))}},8540:function(e,r,t){var n=t(87146),o=t(79788),a=t(70792),i=t(46175),l=t(25350),c=t(80509),u=t(63128),s=t(1975);function f(e){for(var r=1;r<arguments.length;r++){var t=null!=arguments[r]?arguments[r]:{},n=Object.keys(t);"function"==typeof Object.getOwnPropertySymbols&&(n=n.concat(Object.getOwnPropertySymbols(t).filter(function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.forEach(function(r){var n,o,a;n=e,o=r,a=t[r],o in n?Object.defineProperty(n,o,{value:a,enumerable:!0,configurable:!0,writable:!0}):n[o]=a})}return e}var d=(0,n.ZL)()(function(e,r){var t=r.contentWidth;return{dialogContent:{"&.MuiDialogContent-root":{paddingTop:e.spacing(1)},width:t}}});r.Z=function(e){var r=e.open,t=e.onClose,n=e.onCancel,b=e.onConfirm,m=e.labelTitle,p=void 0===m?"Are you sure?":m,y=e.labelCancel,g=e.labelConfirm,v=void 0===g?"Confirm":g,O=e.children,h=e.contentWidth,P=e.confirmDisabled,j=e.cancelDisabled,Z=e.submitting,w=e.dialogPaperClassName,C=e.dialogTitleClassName,S=e.dialogContentClassName,E=e.dialogActionsClassName,x=e.dialogConfirmButtonClassName,D=e.restCancelButtonProps,I=e.restConfirmButtonProps,k=function(e,r){if(null==e)return{};var t,n,o=function(e,r){if(null==e)return{};var t,n,o={},a=Object.keys(e);for(n=0;n<a.length;n++)t=a[n],!(r.indexOf(t)>=0)&&(o[t]=e[t]);return o}(e,r);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(n=0;n<a.length;n++){if(t=a[n],!(r.indexOf(t)>=0))Object.prototype.propertyIsEnumerable.call(e,t)&&(o[t]=e[t])}}return o}(e,["open","onClose","onCancel","onConfirm","labelTitle","labelCancel","labelConfirm","children","contentWidth","confirmDisabled","cancelDisabled","submitting","dialogPaperClassName","dialogTitleClassName","dialogContentClassName","dialogActionsClassName","dialogConfirmButtonClassName","restCancelButtonProps","restConfirmButtonProps"]),A=d({contentWidth:h}),N=A.classes,T=A.cx;return s.createElement(o.Z,f({PaperProps:{className:w},open:r,scroll:"paper",onClose:t},k),p&&s.createElement(a.Z,{className:C},p),O&&s.createElement(i.Z,{className:T(N.dialogContent,S)},O),s.createElement(l.Z,{className:E},n&&s.createElement(c.Z,f({color:"primary","data-testid":"Cancel",disabled:void 0!==j&&j,onClick:n},D),void 0===y?"Cancel":y),s.createElement(c.Z,f({"aria-label":v||"",className:x,color:"primary","data-testid":"Confirm",disabled:void 0!==P&&P,endIcon:void 0!==Z&&Z&&s.createElement(u.Z,{size:15}),onClick:b},I),v)))}},6339:function(e,r,t){var n=t(87146),o=t(44445),a=t(78653),i=t(31274),l=t(18737),c=t(45490),u=t(50708),s=t(1975),f=(0,n.ZL)()(function(e){return{details:{backgroundColor:e.palette.background.paper,padding:e.spacing(1,3,2)}}}),d=(0,o.ZP)(a.Z)(function(e){return{fontSize:e.theme.typography.pxToRem(15),fontWeight:700}}),b=(0,o.ZP)(function(e){return s.createElement(i.Z,function(e){for(var r=1;r<arguments.length;r++){var t=null!=arguments[r]?arguments[r]:{},n=Object.keys(t);"function"==typeof Object.getOwnPropertySymbols&&(n=n.concat(Object.getOwnPropertySymbols(t).filter(function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.forEach(function(r){var n,o,a;n=e,o=r,a=t[r],o in n?Object.defineProperty(n,o,{value:a,enumerable:!0,configurable:!0,writable:!0}):n[o]=a})}return e}({disableGutters:!0,square:!0,elevation:0},e))})(function(e){var r=e.theme;return{"&:before":{display:"none"},"&:not(:last-child)":{borderBottom:"1px solid ".concat(r.palette.divider)},borderBottom:"1px solid ".concat(r.palette.divider),borderLeft:"none",borderRight:"none",borderTop:"none"}}),m=(0,o.ZP)(function(e){return s.createElement(l.Z,e)})(function(e){var r=e.theme;return{"& .MuiAccordionSummary-content":{margin:r.spacing(1)},"& .MuiAccordionSummary-expandIconWrapper":{transform:"rotate(-90deg)"},"& .MuiAccordionSummary-expandIconWrapper.Mui-expanded":{transform:"rotate(0deg)"},backgroundColor:r.palette.background.paper}});r.Z=function(e){var r=e.title,t=e.children,n=f().classes;return s.createElement(b,null,s.createElement(m,{expandIcon:s.createElement(u.default,{color:"action"})},s.createElement(d,null,r)),s.createElement(c.Z,{className:n.details},t))}},64876:function(e,r,t){t.d(r,{h:function(){return O}});var n=t(4822),o=t(87146),a=t(15562),i=t(3587),l=t(45749),c=t(38456),u=t(83409),s=t(56322),f=t(32767),d=t(39782),b=t(6339),m=t(1975);function p(e,r){(null==r||r>e.length)&&(r=e.length);for(var t=0,n=Array(r);t<r;t++)n[t]=e[t];return n}var y=function(e){var r=e.mainPanelWidth;return e.hasSecondaryPanel?r?"".concat(r,"px ").concat(20,"px 1fr"):"1fr ".concat(20,"px 1fr"):"100%"},g=(0,o.ZL)()(function(e,r){var t=r.hasSecondaryPanel,n=r.mainPanelWidth;return{closeIcon:{margin:"auto",width:15},closeSecondaryPanelBar:{alignContent:"center",alignItems:"center",backgroundColor:e.palette.background.paper,borderBottom:"none",borderTop:"none",cursor:"pointer",display:"flex",width:20},container:{display:t?"grid":"block",gridTemplateColumns:y({hasSecondaryPanel:t,mainPanelWidth:n}),height:"100%"},mainPanel:{bottom:0,left:0,overflow:"auto",position:t?"unset":"absolute",right:0,top:0,width:n}}}),v=function(e){var r=e.header,t=e.secondaryPanel,o=e.sections,d=e.onSecondaryPanelClose,p=e.onClose,y=e.onResize,v=e.loading,O=e.mainPanelWidth,h=e.setWidth,P=!(0,n.Z)(t),j=g({hasSecondaryPanel:P,mainPanelWidth:void 0===O?550:O}).classes;return m.createElement(s.Z,{header:r,selectedTab:m.createElement(f.Z,{alignCenter:!0,loading:void 0!==v&&v},m.createElement("div",{className:j.container},m.createElement(a.Z,{className:j.mainPanel},o.map(function(e){var r=e.id,t=e.title,n=e.section;return e.expandable?m.createElement(b.Z,{key:r,title:t},n):m.createElement(i.ZP,{key:r},n)})),P&&m.createElement(l.Z,{square:!0,"aria-label":"Close Secondary Panel",className:j.closeSecondaryPanelBar,onClick:void 0===d?function(){}:d},m.createElement(u.default,{className:j.closeIcon,color:"primary"})),m.createElement(c.Z,{direction:"left",in:P,timeout:{enter:150,exit:50}},m.createElement("div",null,t)))),width:function(){if(P){var e=1120;return null==h||h(e),e}return null==h||h(550),550}(),onClose:void 0===p?function(){}:p,onResize:y})},O=function(e){var r,t=e.memoProps,n=e.sections,o=e.secondaryPanel,a=e.loading,i=function(e,r){if(null==e)return{};var t,n,o=function(e,r){if(null==e)return{};var t,n,o={},a=Object.keys(e);for(n=0;n<a.length;n++)t=a[n],!(r.indexOf(t)>=0)&&(o[t]=e[t]);return o}(e,r);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(n=0;n<a.length;n++){if(t=a[n],!(r.indexOf(t)>=0))Object.prototype.propertyIsEnumerable.call(e,t)&&(o[t]=e[t])}}return o}(e,["memoProps","sections","secondaryPanel","loading"]);return(0,d.n7)({Component:m.createElement(v,function(e){for(var r=1;r<arguments.length;r++){var t=null!=arguments[r]?arguments[r]:{},n=Object.keys(t);"function"==typeof Object.getOwnPropertySymbols&&(n=n.concat(Object.getOwnPropertySymbols(t).filter(function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.forEach(function(r){var n,o,a;n=e,o=r,a=t[r],o in n?Object.defineProperty(n,o,{value:a,enumerable:!0,configurable:!0,writable:!0}):n[o]=a})}return e}({loading:a,secondaryPanel:o,sections:n},i)),memoProps:((function(e){if(Array.isArray(e))return p(e)})(r=void 0===t?[]:t)||function(e){if("undefined"!=typeof Symbol&&null!=e[Symbol.iterator]||null!=e["@@iterator"])return Array.from(e)}(r)||function(e,r){if(e){if("string"==typeof e)return p(e,r);var t=Object.prototype.toString.call(e).slice(8,-1);if("Object"===t&&e.constructor&&(t=e.constructor.name),"Map"===t||"Set"===t)return Array.from(t);if("Arguments"===t||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t))return p(e,r)}}(r)||function(){throw TypeError("Invalid attempt to spread non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}()).concat([n,o,a])})}},73642:function(e,r,t){var n=t(1975),o=n.forwardRef(function(e,r){var t,o,a=e.Content,i=function(e,r){if(null==e)return{};var t,n,o=function(e,r){if(null==e)return{};var t,n,o={},a=Object.keys(e);for(n=0;n<a.length;n++)t=a[n],!(r.indexOf(t)>=0)&&(o[t]=e[t]);return o}(e,r);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(n=0;n<a.length;n++){if(t=a[n],!(r.indexOf(t)>=0))Object.prototype.propertyIsEnumerable.call(e,t)&&(o[t]=e[t])}}return o}(e,["Content"]);return n.createElement(a,(t=function(e){for(var r=1;r<arguments.length;r++){var t=null!=arguments[r]?arguments[r]:{},n=Object.keys(t);"function"==typeof Object.getOwnPropertySymbols&&(n=n.concat(Object.getOwnPropertySymbols(t).filter(function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.forEach(function(r){var n,o,a;n=e,o=r,a=t[r],o in n?Object.defineProperty(n,o,{value:a,enumerable:!0,configurable:!0,writable:!0}):n[o]=a})}return e}({},i),o=(o={itemRef:r},o),Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(o)):(function(e,r){var t=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t.push.apply(t,n)}return t})(Object(o)).forEach(function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(o,e))}),t))});r.Z=o},5e3:function(e,r,t){var n=t(97330),o=t(90996),a=t(78434),i=t(39782),l=t(73642),c=t(1975);function u(e,r){(null==r||r>e.length)&&(r=e.length);for(var t=0,n=Array(r);t<r;t++)n[t]=e[t];return n}function s(e){return function(e){if(Array.isArray(e))return u(e)}(e)||function(e){if("undefined"!=typeof Symbol&&null!=e[Symbol.iterator]||null!=e["@@iterator"])return Array.from(e)}(e)||function(e,r){if(e){if("string"==typeof e)return u(e,r);var t=Object.prototype.toString.call(e).slice(8,-1);if("Object"===t&&e.constructor&&(t=e.constructor.name),"Map"===t||"Set"===t)return Array.from(t);if("Arguments"===t||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t))return u(e,r)}}(e)||function(){throw TypeError("Invalid attempt to spread non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}()}r.Z=function(e){var r,t,u=e.itemId,f=e.memoProps,d=e.additionalProps,b=void 0===d?[]:d,m=e.itemProps,p=e.index,y=function(e,r){if(null==e)return{};var t,n,o=function(e,r){if(null==e)return{};var t,n,o={},a=Object.keys(e);for(n=0;n<a.length;n++)t=a[n],!(r.indexOf(t)>=0)&&(o[t]=e[t]);return o}(e,r);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(n=0;n<a.length;n++){if(t=a[n],!(r.indexOf(t)>=0))Object.prototype.propertyIsEnumerable.call(e,t)&&(o[t]=e[t])}}return o}(e,["itemId","memoProps","additionalProps","itemProps","index"]),g=(0,o.nB)({id:u}),v=g.attributes,O=g.listeners,h=g.setNodeRef,P=g.transform,j=g.isDragging,Z=g.transition,w={opacity:j?"0.7":"1",transform:a.ux.Translate.toString(P),transition:Z};return(0,i.n7)({Component:c.createElement(l.Z,(r=function(e){for(var r=1;r<arguments.length;r++){var t=null!=arguments[r]?arguments[r]:{},n=Object.keys(t);"function"==typeof Object.getOwnPropertySymbols&&(n=n.concat(Object.getOwnPropertySymbols(t).filter(function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.forEach(function(r){var n,o,a;n=e,o=r,a=t[r],o in n?Object.defineProperty(n,o,{value:a,enumerable:!0,configurable:!0,writable:!0}):n[o]=a})}return e}({},y,b),t=(t={attributes:v,index:p,isDragging:j,listeners:O,ref:h,style:w},t),Object.getOwnPropertyDescriptors?Object.defineProperties(r,Object.getOwnPropertyDescriptors(t)):(function(e,r){var t=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t.push.apply(t,n)}return t})(Object(t)).forEach(function(e){Object.defineProperty(r,e,Object.getOwnPropertyDescriptor(t,e))}),r)),memoProps:[p,j,P,(0,n.Z)(m,y)].concat(s(f),s(b),s(Object.values(y)))})}},59737:function(e,r,t){var n=t(1975),o=t(18696),a=t(90996),i=t(72980),l=t(54184),c=t(71982),u=t(4822),s=t(71898),f=t(85824),d=t(16657),b=t(63426),m=t(41274),p=t(76426),y=t(41839),g=t(21294),v=t(5e3),O=t(73642);function h(e,r){(null==r||r>e.length)&&(r=e.length);for(var t=0,n=Array(r);t<r;t++)n[t]=e[t];return n}function P(e){for(var r=1;r<arguments.length;r++){var t=null!=arguments[r]?arguments[r]:{},n=Object.keys(t);"function"==typeof Object.getOwnPropertySymbols&&(n=n.concat(Object.getOwnPropertySymbols(t).filter(function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.forEach(function(r){var n,o,a;n=e,o=r,a=t[r],o in n?Object.defineProperty(n,o,{value:a,enumerable:!0,configurable:!0,writable:!0}):n[o]=a})}return e}function j(e,r){return function(e){if(Array.isArray(e))return e}(e)||function(e,r){var t,n,o=null==e?null:"undefined"!=typeof Symbol&&e[Symbol.iterator]||e["@@iterator"];if(null!=o){var a=[],i=!0,l=!1;try{for(o=o.call(e);!(i=(t=o.next()).done)&&(a.push(t.value),!r||a.length!==r);i=!0);}catch(e){l=!0,n=e}finally{try{!i&&null!=o.return&&o.return()}finally{if(l)throw n}}return a}}(e,r)||function(e,r){if(e){if("string"==typeof e)return h(e,r);var t=Object.prototype.toString.call(e).slice(8,-1);if("Object"===t&&e.constructor&&(t=e.constructor.name),"Map"===t||"Set"===t)return Array.from(t);if("Arguments"===t||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t))return h(e,r)}}(e,r)||function(){throw TypeError("Invalid attempt to destructure non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}()}var Z=function(e){return e.children};r.Z=function(e){var r=e.items,t=e.onDragStart,h=e.onDragEnd,w=e.onDragOver,C=e.collisionDetection,S=e.sortingStrategy,E=e.itemProps,x=e.memoProps,D=void 0===x?[]:x,I=e.additionalProps,k=e.RootComponent,A=void 0===k?Z:k,N=e.Content,T=e.getDisableItemCondition,B=void 0===T?function(){return!1}:T,M=e.getDisableOverItemSortableCondition,R=void 0===M?function(){return!1}:M,L=e.updateSortableItemsOnItemsChange,$=void 0!==L&&L,q=function(){return(0,i.Z)("id",r)},z=j(n.useState(null),2),W=z[0],F=z[1],U=j(n.useState(q()),2),G=U[0],V=U[1],X=(0,o.Dy)((0,o.VT)(o.MA),(0,o.VT)(o.we),(0,o.VT)(o.Lg,{keyboardCodes:{cancel:["Escape"],end:["Space","Enter"],start:["ArrowUp","ArrowDown","ArrowLeft","ArrowRight","Enter","Space"]},scrollBehavior:"smooth"})),_=(0,g.Z)(),H=function(e){return(0,m.Z)((0,p.Z)(e,"id"),r)},Q=H(W);return n.useEffect(function(){if(!(0,s.Z)($))V(q())},[r]),n.createElement(o.LB,{collisionDetection:C,sensors:X,onDragCancel:function(){return F(null)},onDragEnd:function(e){var r,t=(0,l.Z)(["over","id"],e);if((0,c.Z)(u.Z,s.Z)(t)&&(0,c.Z)((0,f.Z)(W),s.Z)(t)&&(0,s.Z)((r=e.over)&&R(r)||!1)){var n=(0,d.Z)(W,G),o=(0,d.Z)(t,G),a=(0,b.Z)(n,o,G);V(a),null==h||h({event:e,items:a})}F(null)},onDragOver:function(e){null==w||w(e)},onDragStart:function(e){null==t||t(e),F((0,l.Z)(["active","id"],e))}},n.createElement(a.Fo,{items:G,strategy:S},n.createElement(A,null,n.createElement(n.Fragment,null,G.map(function(e,r){var t,o,a=H(e);if((0,u.Z)(a))return null;return(0,s.Z)(B(a))&&n.createElement(v.Z,(t=P({Content:N,index:r,itemId:e,itemProps:E,key:e,memoProps:D},(0,y.Z)(E,a)),o=(o={additionalProps:I},o),Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(o)):(function(e,r){var t=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t.push.apply(t,n)}return t})(Object(o)).forEach(function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(o,e))}),t))})))),n.createElement(o.y9,{style:{zIndex:_.zIndex.tooltip}},n.createElement(A,{isInDragOverlay:!0},W?n.createElement(O.Z,P({isDragging:!0,isInDragOverlay:!0,Content:N,title:W},(0,y.Z)(E,Q),I)):null)))}},91062:function(e,r,t){var n=t(1975),o=t(45931),a=t(70119),i=t(85824),l=t(70590),c=t(97580),u=t(71442),s=t(21429),f=t(12889),d=t(76097),b=t(89130),m=t(9691),p=t(87146),y=t(17947),g=t(22850),v=t(7274),O=t(32767),h=t(63363),P=t(48873),j=t(75541),Z=t(15267),w=t(93421),C=t(79065),S=t(22969),E=t(1975);function x(e,r){(null==r||r>e.length)&&(r=e.length);for(var t=0,n=Array(r);t<r;t++)n[t]=e[t];return n}function D(e){for(var r=1;r<arguments.length;r++){var t=null!=arguments[r]?arguments[r]:{},n=Object.keys(t);"function"==typeof Object.getOwnPropertySymbols&&(n=n.concat(Object.getOwnPropertySymbols(t).filter(function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.forEach(function(r){var n,o,a;n=e,o=r,a=t[r],o in n?Object.defineProperty(n,o,{value:a,enumerable:!0,configurable:!0,writable:!0}):n[o]=a})}return e}function I(e,r){return function(e){if(Array.isArray(e))return e}(e)||function(e,r){var t,n,o=null==e?null:"undefined"!=typeof Symbol&&e[Symbol.iterator]||e["@@iterator"];if(null!=o){var a=[],i=!0,l=!1;try{for(o=o.call(e);!(i=(t=o.next()).done)&&(a.push(t.value),!r||a.length!==r);i=!0);}catch(e){l=!0,n=e}finally{try{!i&&null!=o.return&&o.return()}finally{if(l)throw n}}return a}}(e,r)||function(e,r){if(e){if("string"==typeof e)return x(e,r);var t=Object.prototype.toString.call(e).slice(8,-1);if("Object"===t&&e.constructor&&(t=e.constructor.name),"Map"===t||"Set"===t)return Array.from(t);if("Arguments"===t||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t))return x(e,r)}}(e,r)||function(){throw TypeError("Invalid attempt to destructure non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}()}var k=(0,p.ZL)()(function(e){return{filterCard:{alignItems:"center",display:"grid",gridAutoFlow:"column",gridGap:e.spacing(2),gridTemplateColumns:"auto 1fr"}}}),A=function(e){return function(r){return(0,i.Z)(Number(e.id),Number(r.id))}};r.Z=function(e){var r=e.filter,t=k().classes,p=(0,b.useTranslation)().t,x=(0,g.Z)().showSuccessMessage,N=I((0,n.useState)(!1),2),T=N[0],B=N[1],M=(0,v.Z)({request:w.a8}),R=M.sendRequest,L=M.sending,$=(0,v.Z)({request:w.kh}),q=$.sendRequest,z=$.sending,W=I((0,m.useAtom)(S.c_),2),F=W[0],U=W[1],G=I((0,m.useAtom)(S.$h),2),V=G[0],X=G[1],_=(0,m.useSetAtom)(S.qF),H=r.name,Q=r.id,K=a.Ry().shape({name:a.Z_().required(p(Z.Tq_))}),J=(0,o.TA)({enableReinitialize:!0,initialValues:{name:H},onSubmit:function(e){var t,n,o=(t=D({},r),n=(n={name:e.name},n),Object.getOwnPropertyDescriptors?Object.defineProperties(t,Object.getOwnPropertyDescriptors(n)):(function(e,r){var t=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t.push.apply(t,n)}return t})(Object(n)).forEach(function(e){Object.defineProperty(t,e,Object.getOwnPropertyDescriptor(n,e))}),t);R({filter:(0,l.Z)(["id"],o),id:o.id}).then(function(){x(p(Z.cSG)),(0,i.Z)(o.id,F.id)&&U(o);var e=(0,c.Z)(A(r),V);X((0,u.Z)(e,o,V))})},validationSchema:K}),Y=(0,f.Z)((0,i.Z)(!0),[z,L]),ee=(0,d.Z)((0,i.Z)(!0),[J.isValid,J.dirty]),er=function(){if(!!ee)J.submitForm()};return E.createElement("div",{className:t.filterCard},E.createElement(O.Z,{alignCenter:!1,loading:Y,loadingIndicatorSize:24},E.createElement(h.Z,{"aria-label":p(Z.cb1),size:"large",title:p(Z.cb1),onClick:function(){B(!0)}},E.createElement(y.default,{fontSize:"small"}))),E.createElement(P.Z,{transparent:!0,ariaLabel:"".concat(p(Z.mMf),"-").concat(Q,"-").concat(p(Z.xpR)),error:J.errors.name,value:J.values.name,onBlur:er,onChange:J.handleChange("name"),onKeyDown:function(e){13===e.keyCode&&er()}}),T&&E.createElement(j.Z,{open:!0,labelCancel:p(Z.SsP),labelConfirm:p(Z.cb1),labelTitle:p(Z.RS5),onCancel:function(){B(!1)},onConfirm:function(){B(!1),q(r).then(function(){x(p(Z.LNG)),A(r)(F)&&(U(D({},r,C.oW)),_(D({},r,C.oW))),X((0,s.Z)(A(r),V))})}}))}},30424:function(e,r,t){var n=t(87146),o=t(35459),a=t(45749),i=t(91062),l=t(1975);function c(e){for(var r=1;r<arguments.length;r++){var t=null!=arguments[r]?arguments[r]:{},n=Object.keys(t);"function"==typeof Object.getOwnPropertySymbols&&(n=n.concat(Object.getOwnPropertySymbols(t).filter(function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.forEach(function(r){var n,o,a;n=e,o=r,a=t[r],o in n?Object.defineProperty(n,o,{value:a,enumerable:!0,configurable:!0,writable:!0}):n[o]=a})}return e}function u(e,r){return r=null!=r?r:{},Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(r)):(function(e,r){var t=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t.push.apply(t,n)}return t})(Object(r)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(r,t))}),e}var s=(0,n.ZL)()(function(e,r){var t=r.isDragging;return{filterCard:{alignItems:"center",display:"grid",gridGap:e.spacing(2),gridTemplateColumns:"1fr auto",padding:e.spacing(1)},filterCardHandler:{cursor:t?"grabbing":"grab"}}});r.Z=function(e){var r=e.listeners,t=e.attributes,n=e.style,f=e.itemRef,d=e.criterias,b=e.id,m=e.name,p=s({isDragging:e.isDragging}).classes;return l.createElement(a.Z,u(c({square:!0,className:p.filterCard},t),{ref:f,style:n}),l.createElement(i.Z,{filter:{criterias:d,id:b,name:m}}),l.createElement("div",u(c({},r),{className:p.filterCardHandler,role:"button",tabIndex:0}),l.createElement(o.default,null)))}},93530:function(e,r,t){t.r(r);var n=t(1975),o=t(18696),a=t(32888),i=t(9691),l=t(78722),c=t(41274),u=t(85824),s=t(54184),f=t(89130),d=t(87146),b=t(11784),m=t(5037),p=t(78653),y=t(7274),g=t(59737),v=t(64876),O=t(15267),h=t(93421),P=t(22969),j=t(30424),Z=t(1975);function w(e,r){(null==r||r>e.length)&&(r=e.length);for(var t=0,n=Array(r);t<r;t++)n[t]=e[t];return n}function C(e){for(var r=1;r<arguments.length;r++){var t=null!=arguments[r]?arguments[r]:{},n=Object.keys(t);"function"==typeof Object.getOwnPropertySymbols&&(n=n.concat(Object.getOwnPropertySymbols(t).filter(function(e){return Object.getOwnPropertyDescriptor(t,e).enumerable}))),n.forEach(function(r){var n,o,a;n=e,o=r,a=t[r],o in n?Object.defineProperty(n,o,{value:a,enumerable:!0,configurable:!0,writable:!0}):n[o]=a})}return e}function S(e,r){return r=null!=r?r:{},Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(r)):(function(e,r){var t=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t.push.apply(t,n)}return t})(Object(r)).forEach(function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(r,t))}),e}var E=(0,d.ZL)()(function(e){return{container:{width:"100%"},filters:{display:"grid",gridAutoFlow:"row",gridGap:e.spacing(3),gridTemplateRows:"1fr",width:"100%"},header:{alignItems:"center",display:"flex",height:"100%",justifyContent:"center"},loadingIndicator:{height:e.spacing(1),marginBottom:e.spacing(1),width:"100%"}}});r.default=function(){var e,r,t=E().classes,d=(0,f.useTranslation)().t,x=(0,y.Z)({request:h.Q$}),D=x.sendRequest,I=x.sending;var k=(e=(0,i.useAtom)(P.$h),r=2,function(e){if(Array.isArray(e))return e}(e)||function(e,r){var t,n,o=null==e?null:"undefined"!=typeof Symbol&&e[Symbol.iterator]||e["@@iterator"];if(null!=o){var a=[],i=!0,l=!1;try{for(o=o.call(e);!(i=(t=o.next()).done)&&(a.push(t.value),!r||a.length!==r);i=!0);}catch(e){l=!0,n=e}finally{try{!i&&null!=o.return&&o.return()}finally{if(l)throw n}}return a}}(e,2)||function(e,r){if(e){if("string"==typeof e)return w(e,r);var t=Object.prototype.toString.call(e).slice(8,-1);if("Object"===t&&e.constructor&&(t=e.constructor.name),"Map"===t||"Set"===t)return Array.from(t);if("Arguments"===t||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t))return w(e,r)}}(e,r)||function(){throw TypeError("Invalid attempt to destructure non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}()),A=k[0],N=k[1],T=(0,i.useSetAtom)(P.TU),B=(0,l.Z)(function(e){var r=e.id;return S(C({},function(e,r){if(null==e)return{};var t,n,o=function(e,r){if(null==e)return{};var t,n,o={},a=Object.keys(e);for(n=0;n<a.length;n++)t=a[n],!(r.indexOf(t)>=0)&&(o[t]=e[t]);return o}(e,r);if(Object.getOwnPropertySymbols){var a=Object.getOwnPropertySymbols(e);for(n=0;n<a.length;n++){if(t=a[n],!(r.indexOf(t)>=0))Object.prototype.propertyIsEnumerable.call(e,t)&&(o[t]=e[t])}}return o}(e,["id"])),{id:"".concat(r)})},A),M=(0,n.useCallback)(function(e){var r=e.children;return Z.createElement(b.Z,{spacing:2},r)},[]),R=[{expandable:!1,id:"edit",section:Z.createElement("div",{className:t.container},Z.createElement("div",{className:t.loadingIndicator},I&&Z.createElement(m.Z,{style:{width:"100%"}})),Z.createElement(g.Z,{updateSortableItemsOnItemsChange:!0,Content:j.Z,RootComponent:M,collisionDetection:o.Xo,itemProps:["criterias","id","name"],items:B,sortingStrategy:a.U2,onDragEnd:function(e){var r=e.items,t=e.event;N((0,l.Z)(function(e){return S(C({},(0,c.Z)(function(r){return(0,u.Z)(Number(r.id),Number(e))},A)),{order:r.indexOf(e)})},r)),D({id:(0,s.Z)(["active","id"],t),order:(0,s.Z)(["over","data","current","sortable","index"],t)+1})}}))}],L=Z.createElement("div",{className:t.header},Z.createElement(p.Z,{align:"center",variant:"h6"},d(O.Qq9)));return Z.createElement(v.h,{header:L,memoProps:[A],sections:R,onClose:function(){T(!1)}})}}}]);
//# sourceMappingURL=4269.4194f32d11125a91b1e7.aa23b5efb4d24d237055.e8e7040a846d504f76a8.js.map