<?php

/*
 * Copyright 2005 - 2023 Centreon (https://www.centreon.com/)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * For more information : contact@centreon.com
 *
 */

declare(strict_types = 1);

namespace Core\Host\Infrastructure\API\FindHosts;

use Centreon\Application\Controller\AbstractController;
use Core\Host\Application\UseCase\FindHosts\FindHosts;
use Symfony\Component\HttpFoundation\Response;

final class FindHostsController extends AbstractController
{
    public function __invoke(
        FindHosts $useCase,
        FindHostsOnPremPresenter $onPremPresenter,
        FindHostsSaasPresenter $saasPresenter,
        bool $isCloudPlatform
    ): Response {
        $this->denyAccessUnlessGrantedForApiConfiguration();

        $presenter = $isCloudPlatform ? $saasPresenter : $onPremPresenter;

        $useCase($presenter);

        return $presenter->show();
    }
}
