<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Symfony\Component\Security\Core\Signature\Exception;

use Symfony\Component\Security\Core\Exception\RuntimeException;

/**
 * @author Wouter de Jong <wouter@wouterj.nl>
 */
class ExpiredSignatureException extends RuntimeException
{
}
