export const labelDiscard = 'Discard';
export const labelResolve = 'Resolve';
export const labelIfYouClickOnDiscard =
  'If you click on Discard, your changes will not be saved.';
export const labelThereAreErrorsInTheForm = 'There are errors in the form';
export const labelDoYouWantToQuitWithoutResolving =
  'Do you want to quit the form without resolving the errors?';
export const labelDoYouWantToSaveChanges = 'Do you want to save the changes?';
export const labelDoYouWantToResolveErrors =
  'Do you want to resolve the errors?';
export const labelSave = 'Save';

export default {
  labelDiscard,
  labelDoYouWantToQuitWithoutResolving,
  labelDoYouWantToResolveErrors,
  labelDoYouWantToSaveChanges,
  labelIfYouClickOnDiscard,
  labelResolve,
  labelSave,
  labelThereAreErrorsInTheForm
};
