"use strict";(self.webpackChunkcentreon=self.webpackChunkcentreon||[]).push([["5304"],{88428:function(n,t,e){var r=e(9691),o=e(52573),i=(0,r.atom)(o.T3);t.Z=i},19814:function(n,t,e){var r=e(9691),o=e(52573),i=(0,r.atom)(o.kn);t.Z=i},77913:function(n,t,e){e.d(t,{F:function(){return r}});var r=(0,e(9691).atom)(navigator.language.slice(0,2))},34393:function(n,t,e){var r=(0,e(9691).atom)(!1);t.Z=r},52573:function(n,t,e){e.d(t,{Bt:function(){return o},Ml:function(){return c},T3:function(){return m},YO:function(){return a},kn:function(){return u},yK:function(){return i}});var r=e(3772),o={alias:"",canManageApiTokens:!1,default_page:"/monitoring/resources",id:void 0,isAdmin:void 0,isExportButtonEnabled:!1,locale:null,name:"",themeMode:r.hY.light,timezone:Intl.DateTimeFormat().resolvedOptions().timeZone,use_deprecated_pages:!1,user_interface_density:r.jq.compact},i=!1,u={actions:{host:{acknowledgement:!1,check:!1,comment:!1,disacknowledgement:!1,downtime:!1,submit_status:!1},service:{acknowledgement:!1,check:!1,comment:!1,disacknowledgement:!1,downtime:!1,submit_status:!1}}},a={duration:3600,fixed:!0,with_services:!1},c=15,m={force_active_checks:!1,notify:!0,persistent:!1,sticky:!1,with_services:!0}},50613:function(n,t,e){var r=e(9691),o=e(52573),i=(0,r.atom)(o.YO);t.Z=i},78390:function(n,t,e){e.r(t),e.d(t,{DashboardGlobalRole:function(){return d.ex},ListingVariant:function(){return d.jq},ThemeMode:function(){return d.hY},acknowledgementAtom:function(){return m.Z},aclAtom:function(){return o.Z},browserLocaleAtom:function(){return h.F},cloudServicesAtom:function(){return c.Z},downtimeAtom:function(){return i.Z},featureFlagsDerivedAtom:function(){return v.z},isResourceStatusFullSearchEnabledAtom:function(){return Z.B},platformFeaturesAtom:function(){return v.S},platformNameAtom:function(){return s.Z},refreshIntervalAtom:function(){return u.Z},resourceStorageOptimizationModeAtom:function(){return f.Z},statisticsRefreshIntervalAtom:function(){return a.Z},userAtom:function(){return r.Z},userPermissionsAtom:function(){return l.Z}});var r=e(3234),o=e(19814),i=e(50613),u=e(80284),a=e(3077),c=e(34393),m=e(88428),f=e(49269),s=e(55296),l=e(47765),d=e(3772),v=e(28546),h=e(77913),Z=e(48776)},48776:function(n,t,e){e.d(t,{B:function(){return r}});var r=(0,e(9691).atom)(!1)},28546:function(n,t,e){e.d(t,{S:function(){return o},z:function(){return i}});var r=e(9691),o=(0,r.atom)(null),i=(0,r.atom)(function(n){var t=n(o);return null==t?void 0:t.featureFlags})},55296:function(n,t,e){var r=(0,e(9691).atom)(null);t.Z=r},80284:function(n,t,e){var r=e(9691),o=e(52573),i=(0,r.atom)(o.Ml);t.Z=i},49269:function(n,t,e){var r=e(9691),o=e(52573),i=(0,r.atom)(o.yK);t.Z=i},3077:function(n,t,e){var r=e(9691),o=e(52573),i=(0,r.atom)(o.Ml);t.Z=i},3772:function(n,t,e){var r,o,i,u,a,c;e.d(t,{ex:function(){return o},hY:function(){return i},jq:function(){return r}}),(u=r||(r={})).compact="compact",u.extended="extended",(a=o||(o={})).administrator="administrator",a.creator="creator",a.viewer="viewer",(c=i||(i={})).dark="dark",c.light="light"},3234:function(n,t,e){var r=e(9691),o=e(52573),i=(0,r.atom)(o.Bt);t.Z=i},47765:function(n,t,e){var r=(0,e(9691).atom)(null);t.Z=r}}]);
//# sourceMappingURL=5304.d25d9ccfa1f3410bb3fc.9affb5ed93b8104386e3.e8e7040a846d504f76a8.js.map