<?php
$help = array();

$help["alias"] = dgettext("help", "The alias is a longer name or description for the vendor/manufacturer.");
$help["name"] = dgettext("help", "The name is used to identify the vendor/manufacturer with a short name.");
$help["description"] = dgettext(
    "help",
    "Use this for an optional description or comments about the vendor/manufacturer."
);
