"use strict";(self.webpackChunkcentreon=self.webpackChunkcentreon||[]).push([["9091"],{99237:function(e,t,n){n.r(t);var a=n(87146),l=n(4822),i=n(89130),c=n(78653),m=n(52708),r=n(9161),o=n(10548),s=n(1975),u=(0,a.ZL)()({loginElementItem:{display:"inline",verticalAlign:"middle"}});t.default=function(e){var t=e.loginPageCustomisation,n=u().classes,a=(0,i.useTranslation)().t,g=!(0,l.Z)(t.iconSource);return s.createElement("div",{id:"loginHeader"},g&&s.createElement(m.Z,{alt:a(o.mP),className:n.loginElementItem,fallback:s.createElement(r.Z,{height:50,width:250}),height:50,imagePath:t.iconSource||""}),(null==t?void 0:t.platformName)&&s.createElement(c.Z,{className:n.loginElementItem,variant:"h4"},t.platformName))}}}]);
//# sourceMappingURL=9091.ecf4437465e485b701a6.e334878ca8adb5a97f35.e8e7040a846d504f76a8.js.map