"use strict";(self.webpackChunkcentreon=self.webpackChunkcentreon||[]).push([["3575"],{59088:function(e,n,t){t.r(n);var r=t(89130),u=t(62359),a=t(34115),o=t(1975);n.default=function(){var e=(0,r.useTranslation)().t;return o.createElement(u.E,{message:e(a.E),title:e(a.W)})}},34115:function(e,n,t){t.d(n,{E:function(){return r},W:function(){return u}});var r="You are not allowed to see this page",u="Lost in space?"}}]);
//# sourceMappingURL=3575.686d6bc94fede0354c9e.ca0548c3a09f5f62559a.e8e7040a846d504f76a8.js.map