"use strict";(self.webpackChunkcentreon=self.webpackChunkcentreon||[]).push([["4673"],{28110:function(e,t,n){n.r(t);var r=n(1975),o=n(83004),l=n(78540),a=n(2344),i=n(1975),u=(0,r.lazy)(function(){return Promise.all([n.e("4140"),n.e("8323"),n.e("6139"),n.e("5469"),n.e("6034"),n.e("6790"),n.e("5446"),n.e("5202"),n.e("5672"),n.e("2022"),n.e("6794"),n.e("4855"),n.e("5279"),n.e("4587"),n.e("6916"),n.e("1309"),n.e("1683"),n.e("166"),n.e("7933"),n.e("7932"),n.e("2478"),n.e("1504"),n.e("5710"),n.e("3962"),n.e("1998"),n.e("7820"),n.e("3388"),n.e("1251"),n.e("2633"),n.e("4030"),n.e("761"),n.e("696"),n.e("5828"),n.e("2504"),n.e("5058"),n.e("828"),n.e("633"),n.e("5347"),n.e("7563"),n.e("5209"),n.e("9588"),n.e("709"),n.e("4656"),n.e("8145"),n.e("2622"),n.e("7288"),n.e("5857"),n.e("681"),n.e("9830"),n.e("4311"),n.e("1895"),n.e("3906"),n.e("3849"),n.e("3269"),n.e("2483"),n.e("361"),n.e("3239"),n.e("2189"),n.e("1927")]).then(n.bind(n,44274))});t.default=function(){return i.createElement(r.Suspense,{fallback:i.createElement(l.Z,null)},i.createElement(o.Routes,null,i.createElement(o.Route,{element:i.createElement(a.Z,null),path:"/main.php/*"}),i.createElement(o.Route,{element:i.createElement(u,null),path:"/*"})))}},2344:function(e,t,n){var r=n(1975),o=n(83004),l=n(85824),a=n(4822),i=n(54157),u=n(40281),c=n(82298),d=n(78540),m=n(1975);function f(e,t){(null==t||t>e.length)&&(t=e.length);for(var n=0,r=Array(t);n<t;n++)r[n]=e[n];return r}t.Z=function(){var e,t,n=(e=(0,r.useState)(!0),t=2,function(e){if(Array.isArray(e))return e}(e)||function(e,t){var n,r,o=null==e?null:"undefined"!=typeof Symbol&&e[Symbol.iterator]||e["@@iterator"];if(null!=o){var l=[],a=!0,i=!1;try{for(o=o.call(e);!(a=(n=o.next()).done)&&(l.push(n.value),!t||l.length!==t);a=!0);}catch(e){i=!0,r=e}finally{try{!a&&null!=o.return&&o.return()}finally{if(i)throw r}}return l}}(e,2)||function(e,t){if(e){if("string"==typeof e)return f(e,t);var n=Object.prototype.toString.call(e).slice(8,-1);if("Object"===n&&e.constructor&&(n=e.constructor.name),"Map"===n||"Set"===n)return Array.from(n);if("Arguments"===n||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))return f(e,t)}}(e,t)||function(){throw TypeError("Invalid attempt to destructure non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}()),v=n[0],s=n[1],h=(0,o.useLocation)(),y=(0,o.useNavigate)(),p=(0,c.O)().toggleFullscreen,E=function(e){var t=e.detail.href;window.history.pushState(null,t,t)},g=function(e){var t,n,r,o,a,i;if(!((0,u.Z)(null===(r=document.getElementById("main-content"))||void 0===r?void 0:null===(n=r.contentWindow)||void 0===n?void 0:null===(t=n.document.activeElement)||void 0===t?void 0:t.tagName,["INPUT","TEXTAREA"])||(0,l.Z)(null===(i=document.getElementById("main-content"))||void 0===i?void 0:null===(a=i.contentWindow)||void 0===a?void 0:null===(o=a.document.activeElement)||void 0===o?void 0:o.getAttribute("contenteditable"),"true"))&&!!(0,l.Z)(e.code,"KeyF"))p(document.querySelector("body"))};(0,r.useEffect)(function(){var e,t;return window.addEventListener("react.href.update",E,!1),null===(t=document.getElementById("main-content"))||void 0===t||null===(e=t.contentWindow)||void 0===e||e.addEventListener("keypress",g,!1),function(){var e,t;window.removeEventListener("react.href.update",E),null===(t=document.getElementById("main-content"))||void 0===t||null===(e=t.contentWindow)||void 0===e||e.removeEventListener("keypress",g)}},[]);var b=h.search,w=h.hash;return m.createElement(m.Fragment,null,v&&m.createElement(d.Z,null),m.createElement("iframe",{frameBorder:"0",id:"main-content",name:"main-content",scrolling:"yes",src:"./main.get.php".concat((b||"")+(w||"")),style:{height:"100%",width:"100%"},title:"Main Content",onLoad:function(){var e,t;s(!1),null===(t=document.getElementById("main-content"))||void 0===t||null===(e=t.contentWindow)||void 0===e||e.document.querySelectorAll("a").forEach(function(e){e.addEventListener("click",function(e){var t=e.target.getAttribute("href"),n=e.target.getAttribute("target");if((0,l.Z)(n,"_blank"))return;if(e.preventDefault(),(0,a.Z)(t))return;var r=(0,i.Z)("./","",t);if(!(0,l.Z)(r,"#")&&!!r.match(/^main.php/))y("/".concat(r),{replace:!0})},{once:!0})})}}))}},54157:function(e,t,n){var r=(0,n(50632).Z)(function(e,t,n){return n.replace(e,t)});t.Z=r}}]);
//# sourceMappingURL=4673.299709cfd1b12038dfd4.692ef38de6af49c3d80a.e8e7040a846d504f76a8.js.map