<?php
$help = array();

$help["vendor"] = dgettext("help", "Choose a vendor from the list. The vendor must have been created beforehand.");
$help["filename"] = dgettext(
    "help",
    "Choose a local MIB file containing SNMP trap definitions to upload and import. " .
    "The file will be parsed with snmpttconvertmib."
);
