<?php
$help = array();

/**
 * General Information
 */
$help["tip_template_name"] = dgettext("help", "Name of graph template.");
$help["tip_vertical_label"] = dgettext("help", "Vertical Label (Y-axis).");
$help["tip_width"] = dgettext("help", "Width of grid. Used to export the chart.");
$help["tip_height"] = dgettext("help", "Height of grid. Used to export the chart.");
$help["tip_lower_limit"] = dgettext("help", "Lower limit of grid.");
$help["tip_upper_limit"] = dgettext("help", "Upper limit of grid.");
$help["tip_base"] = dgettext("help", "Base value.");

/**
 * Legend
 */
$help["tip_scale_graph_values"] = dgettext("help", "Enables auto scale of graph.");
$help["tip_default_centreon_graph_template"] = dgettext("help", "Set as default graph template.");
$help["tip_comments"] = dgettext("help", "Comments regarding the graph template.");
