"use strict";(self.webpackChunkcentreon=self.webpackChunkcentreon||[]).push([["3906"],{23193:function(t,e,n){n.d(e,{US:function(){return d}});var a=n(4587),r=n.n(a),c=n(77115),o=n(4822),i=n(7260),u=n(85824),l=n(543),s=n(48176),f=n(98497),m=n(87013),b=function(t){if(t<1e3)return"".concat(r()(t).format("0.[00]a")," ms");var e=r()(t/1e3).format("0.[00]a");return"".concat(e," seconds")};!function(){try{r().register("format","milliseconds",{format:function(t){return b(t)},regexps:{format:/(ms)/,unformat:/(ms)/},unformat:function(){return""}})}catch(t){return null}}();var Z=function(t){var e=t.unit,n=t.base;return["B","bytes","bytespersecond","B/s","B/sec","o","octets","b/s","b"].includes(e)||1024===Number(n)},v=function(t){var e=t.value,n=t.unit,a=t.base;if((0,o.Z)(e))return null;var c=Z({base:void 0===a?1e3:a,unit:n}),m=(0,i.Z)([[(0,u.Z)("ms"),(0,l.Z)(" ms")],[s.Z,(0,l.Z)(c?" ib":"a")]])(n),b=r()(Math.abs(e)).format("0.[00]".concat(m)).replace(/iB/g,n);return(0,f.Z)(e,0)?"-".concat(b):b},d=function(t){var e=t.value,n=t.unit,a=t.base,c=void 0===a?1e3:a,i=t.isRaw;if((0,o.Z)(e))return null;var l=Z({base:c,unit:n});if(void 0!==i&&i){var s=(0,u.Z)("%",n)?n:" ".concat(n);return"".concat(e).concat(s)}if((0,u.Z)("%",n))return"".concat(r()(Math.abs(e)).format("0.[00]"),"%");var f=v({base:c,unit:n,value:e});return l||!n||(0,u.Z)(n,"ms")?f:"".concat(f," ").concat(n)};(0,c.Z)(m.Z).center},71817:function(t,e,n){var a,r,c,o,i,u;n.d(e,{LI:function(){return c}}),(o=a||(a={})).center="center",o.left="left",o.right="right",(i=r||(r={})).bottom="bottom",i.center="center",i.top="top",(u=c||(c={})).column="column",u.row="row"},38661:function(t,e,n){n.d(e,{MD:function(){return u},Zn:function(){return r}});var a,r,c=n(24341),o=n.n(c),i=n(59191);"".concat("L"," ").concat("LT"),(a=r||(r={})).end="end",a.start="start";var u={dateTimeFormat:"LT",getStart:function(){return o()(Date.now()).subtract(24,"hour").toDate()},id:"last_24_h",largeName:i.QR,name:i.SE,timelineEventsLimit:20};i.hO,i.fY,i.hr,i.L$},86618:function(t,e,n){var a=n(4822),r=n(54082),c=n(87146),o=n(22833),i=n(7708),u=n(63363),l=n(77283),s=n(1975),f=(0,c.ZL)()(function(t){return{avatar:{backgroundColor:t.palette.primary.main,fontSize:t.typography.body2.fontSize,height:t.spacing(2),width:t.spacing(2)},button:{padding:0}}});e.Z=function(t){var e=t.endpoint,n=t.title,c=t.icon,m=t.avatarTitle,b=f().classes,Z=(0,a.Z)(e)||(0,r.Z)(e),v=(0,a.Z)(n)||(0,r.Z)(n);return Z&&v?null:Z?s.createElement(l.Z,null,s.createElement(o.Z,{className:b.avatar,title:n},s.createElement(i.Z,null,m))):s.createElement(l.Z,null,s.createElement("a",{href:e,onClick:function(t){t.stopPropagation()}},s.createElement(u.Z,{ariaLabel:n,className:b.button,"data-testid":n||e,size:"large",title:n||e,onClick:function(){return null}},c)))}},5285:function(t,e,n){var a=n(87146),r=n(78653),c=n(42923),o=n(40408),i=n(1975),u=(0,a.ZL)()(function(t){return{containerLabel:{padding:t.spacing(.5)},label:{alignItems:"center",justifyContent:"center",lineHeight:1}}});e.Z=function(t){var e=t.label,n=u().classes;return i.createElement(c.ZP,{classes:{label:n.containerLabel},label:i.createElement(r.Z,{className:n.label},e),severityCode:o.Z.None})}}}]);
//# sourceMappingURL=3906.a2410bd9ea5b3b3a19f2.6d039bd89766b17b768a.e8e7040a846d504f76a8.js.map