import { SvgIconProps } from '@mui/material';

import { DowntimeIcon } from '../../../Icon/DowntimeIcon';

const Downtime = (props: SvgIconProps): JSX.Element => (
  <DowntimeIcon {...props} />
);

export default Downtime;
