<?php

$help = array();
$help['img_name'] = dgettext('help', 'Enter a new name for the image.');
$help['img_dir'] = dgettext(
    'help',
    'Enter an already existing or a new directory to add the uploads '
    . 'to it. A non-existent directory name will be created first.'
);
$help['img_file'] = dgettext(
    'help',
    'Select a local file to upload. You can upload jpg, png, gif and '
    . 'gd2 files. Multiple images can be uploaded together inside '
    . 'archives like zip, tar, tar.gz or tar.bz2.'
);

/**
 * formDirectory.ihtml
 */

$help['tip_destination_directory'] = dgettext('help', 'Destination directory.');
$help['tip_images'] = dgettext('help', 'Images.');
