<?php
$conf_centreon['centreon_dir'] = "/usr/share/centreon/";
$conf_centreon['centreon_etc'] = "/etc/centreon/";
$conf_centreon['centreon_dir_www'] = "/usr/share/centreon/www/";
$conf_centreon['centreon_dir_rrd'] = "/var/lib/centreon";
$conf_centreon['centreon_log'] = "/var/log/centreon";
$conf_centreon['centreon_cachedir'] = "/var/cache/centreon/";
$conf_centreon['centreon_varlib'] = "/var/lib/centreon";
$conf_centreon['centreon_group'] = "centreon";
$conf_centreon['centreon_user'] = "centreon";
$conf_centreon['rrdtool_dir'] = "/usr/bin/rrdtool";
$conf_centreon['apache_user'] = "apache";
$conf_centreon['apache_group'] = "apache";
$conf_centreon['mail'] = "/bin/mail";
$conf_centreon['broker_user'] = "centreon-broker";
$conf_centreon['broker_group'] = "centreon-broker";
$conf_centreon['broker_etc'] = "/etc/centreon-broker";
$conf_centreon['monitoring_user'] = "centreon-engine";
$conf_centreon['monitoring_group'] = "centreon-engine";
$conf_centreon['monitoring_etc'] = "/etc/centreon-engine";
$conf_centreon['monitoring_binary'] = "/usr/sbin/centengine";
$conf_centreon['monitoring_varlog'] = "/var/log/centreon-engine";
$conf_centreon['plugin_dir'] = "/usr/lib/nagios/plugins";
$conf_centreon['centreon_engine_connectors'] = "/usr/lib64/centreon-connector";
$conf_centreon['centreon_engine_lib'] = "/usr/lib64/centreon-engine";
$conf_centreon['centreonbroker_cbmod'] = "/usr/lib64/nagios/cbmod.so";
$conf_centreon['centreon_plugins'] = "/usr/lib/centreon/plugins";
