"use strict";(self.webpackChunkcentreon=self.webpackChunkcentreon||[]).push([["6616"],{60398:function(e,n,t){t.r(n);var u=t(89130),r=t(62359),a=t(12950),c=t(1975);n.default=function(){var e=(0,u.useTranslation)().t;return c.createElement(r.E,{message:e(a.pi),title:"404"})}},12950:function(e,n,t){t.d(n,{pi:function(){return u}});var u="This page could not be found"}}]);
//# sourceMappingURL=6616.e923f99bd9a6b98a0c09.6bfc7202148308a142cc.e8e7040a846d504f76a8.js.map