.bar {
    position: relative;
    margin: 0 6px;
    padding: 2px;
    height: 15px;
    text-align: center;
    border: 1px grey solid;
    border-radius: 4px;
}
.bar .progress-ok {
    background-color: #88B922;
    height: 100%;
    display: block;
    border-radius: 4px;
}
.bar .progress-warning {
    background-color: #FD9B27;
    height: 100%;
    display: block;
    border-radius: 4px;
}
.bar .progress-critical {
    background-color: #FF6666;
    height: 100%;
    display: block;
    color: #e00b3d;
    border-radius: 4px;
}
.bar .progress-unknown {
    background-color: #757575;
    height: 100%;
    display: block;
    border-radius: 4px;
}
.styleTable .stat {
    width : 30%;
}
.bar .percent {
    position: absolute;
    top: 3px;
    line-height: 15px;
    border-radius: 4px;
}
.index {
    width: 15px;
    text-align: center;
}
.title {
    text-align : center;
    line-height: 45px;
}
