@import url("http://simile.mit.edu/styles/default.css");

body {
    margin-bottom: 20em;
}

table {
    font-size: 100%;
}

.timeline-default {
    font-family: Trebuchet MS, Helvetica, Arial, sans serif;
    font-size: 8pt;
    border: 1px solid #aaa;
}
