<?php

/*
 * Copyright 2005 - 2025 Centreon (https://www.centreon.com/)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * For more information : contact@centreon.com
 *
 */

declare(strict_types=1);

namespace Adaptation\Database\Connection\Enum;

/**
 * Enum.
 *
 * @class   ConnectionDriverEnum
 */
enum ConnectionDriverEnum: string
{
    case DRIVER_PDO_MYSQL = 'pdo_mysql';
    case DRIVER_PDO_POSTGRESQL = 'pdo_pgsql';
    case DRIVER_PDO_SQLITE = 'pdo_sqlite';
    case DRIVER_PDO_ORACLE = 'pdo_oci';
    case DRIVER_PDO_MICROSOFT_SQL_SERVER = 'pdo_sqlsrv';
    case DRIVER_MYSQLI = 'mysqli';
    case DRIVER_POSTGRESQL = 'pgsql';
    case DRIVER_MICROSOFT_SQL_SERVER = 'sqlsrv';
    case DRIVER_SQLITE3 = 'sqlite3';
}
